@@ -1,3 +1,4 @@
+#include <getopt.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -90,7 +91,17 @@ const owl_cmd commands_to_init[]
 	      "Binds a key sequence to a command within a keymap.\n"
 	      "Use 'show keymaps' to see the existing keymaps.\n"
 	      "Key sequences may be things like M-C-t or NPAGE.\n\n"
-	      "Ex.: bindkey recv C-b command zwrite -c barnowl"),
+	      "Ex.: bindkey recv C-b command zwrite -c barnowl\n"
+              "Ex.: bindkey recv m command start-command zwrite -c my-class -i \n\n"
+              "SEE ALSO: unbindkey, start-command"),
+
+  OWLCMD_ARGS("unbindkey", owl_command_unbindkey, OWL_CTX_ANY,
+	      "removes a binding in a keymap",
+	      "bindkey <keymap> <keyseq>",
+	      "Removes a binding of a key sequence within a keymap.\n"
+	      "Use 'show keymaps' to see the existing keymaps.\n"
+	      "Ex.: unbindkey recv H\n\n"
+              "SEE ALSO: bindkey"),
 
   OWLCMD_ARGS("zwrite", owl_command_zwrite, OWL_CTX_INTERACTIVE,
 	      "send a zephyr",
@@ -582,7 +593,7 @@ const owl_cmd commands_to_init[]
 
   OWLCMD_ARGS("smartnarrow", owl_command_smartnarrow, OWL_CTX_INTERACTIVE,
 	      "view only messages similar to the current message",
-	      "smartnarrow [-i | --instance]",
+	      "smartnarrow [-i | --instance]  [-r | --relatde]",
 	      "If the curmsg is a personal message narrow\n"
 	      "   to the conversation with that user.\n"
 	      "If the curmsg is a <MESSAGE, foo, *>\n"
@@ -590,7 +601,9 @@ const owl_cmd commands_to_init[]
 	      "If the curmsg is a class message, narrow\n"
 	      "    to the class.\n"
 	      "If the curmsg is a class message and '-i' is specified\n"
-	      "    then narrow to the class and instance.\n"),
+	      "    then narrow to the class and instance.\n"
+	      "If '-r' or '--related' is specified, behave as though the\n"
+              "    'narrow-related' variable was inverted."),
 
   OWLCMD_ARGS("smartfilter", owl_command_smartfilter, OWL_CTX_INTERACTIVE,
 	      "returns the name of a filter based on the current message",
@@ -620,6 +633,8 @@ const owl_cmd commands_to_init[]
 	      "view to it.\n\n"
 	      "SEE ALSO: filter, view, viewclass\n"),
   OWLCMD_ALIAS("vu", "viewuser"),
+  OWLCMD_ALIAS("viewperson", "viewuser"),
+  OWLCMD_ALIAS("vp", "viewuser"),
 
   OWLCMD_ARGS("show", owl_command_show, OWL_CTX_INTERACTIVE,
 	      "show information",
@@ -896,15 +911,8 @@ const owl_cmd commands_to_init[]
 		  "insert the current text from the kill buffer",
 		  "", ""),
 
-  OWLCMD_VOID_CTX("editline:done", owl_command_editline_done, 
-		  OWL_CTX_EDITLINE,
-		  "completes the command (eg, executes command being composed)",
-		  "", ""),
-
-  OWLCMD_VOID_CTX("editresponse:done", owl_command_editresponse_done, 
-		  OWL_CTX_EDITRESPONSE,
-		  "completes the response to a question",
-		  "", ""),
+  OWLCMD_ALIAS   ("editline:done", "edit:done"),
+  OWLCMD_ALIAS   ("editresponse:done", "edit:done"),
 
   OWLCMD_VOID_CTX("edit:move-up-line", owl_editwin_key_up, 
 		  OWL_CTX_EDITMULTI,
@@ -917,8 +925,8 @@ const owl_cmd commands_to_init[]
 		  "", ""),
 
   OWLCMD_VOID_CTX("edit:done", owl_command_edit_done, 
-		  OWL_CTX_EDITMULTI,
-		  "completes the command (eg, sends message being composed)",
+		  OWL_CTX_EDIT,
+		  "Finishes entering text in the editwin.",
 		  "", ""),
 
   OWLCMD_VOID_CTX("edit:done-or-delete", owl_command_edit_done_or_delete, 
@@ -1235,10 +1243,10 @@ char *owl_command_next(int argc, const char *const *argv, const char *buff)
       filter = owl_strdup(argv[2]);
       argc-=2; argv+=2; 
     } else if (argc>=2 && !strcmp(argv[1], "--smart-filter")) {
-      filter = owl_function_smartfilter(0);
+      filter = owl_function_smartfilter(0, 0);
       argc-=2; argv+=2; 
     } else if (argc>=2 && !strcmp(argv[1], "--smart-filter-instance")) {
-      filter = owl_function_smartfilter(1);
+      filter = owl_function_smartfilter(1, 0);
       argc-=2; argv+=2; 
     } else {
       owl_function_makemsg("Invalid arguments to command 'next'.");
@@ -1265,10 +1273,10 @@ char *owl_command_prev(int argc, const char *const *argv, const char *buff)
       filter = owl_strdup(argv[2]);
       argc-=2; argv+=2; 
     } else if (argc>=2 && !strcmp(argv[1], "--smart-filter")) {
-      filter = owl_function_smartfilter(0);
+      filter = owl_function_smartfilter(0, 0);
       argc-=2; argv+=2; 
     } else if (argc>=2 && !strcmp(argv[1], "--smart-filter-instance")) {
-      filter = owl_function_smartfilter(1);
+      filter = owl_function_smartfilter(1, 0);
       argc-=2; argv+=2;  
    } else {
       owl_function_makemsg("Invalid arguments to command 'prev'.");
@@ -1284,17 +1292,43 @@ char *owl_command_smartnarrow(int argc, const char *const *argv, const char *buf
 {
   char *filtname = NULL;
 
-  if (argc == 1) {
-    filtname = owl_function_smartfilter(0);
-  } else if (argc == 2 && (!strcmp(argv[1], "-i") || !strcmp(argv[1], "--instance"))) {
-    filtname = owl_function_smartfilter(1);
-  } else {
-    owl_function_makemsg("Wrong number of arguments for %s", argv[0]);    
+  char opt;
+  int instance = 0, related = 0, i;
+  const char **tmp_argv = owl_malloc(sizeof(char *) * argc);
+
+  for (i = 0; i < argc; i++)
+    tmp_argv[i] = argv[i];
+
+  static const struct option options[] = {
+    {"instance", 0, 0, 'i'},
+    {"related",  0, 0, 'r'},
+    {NULL,       0, 0, 0}};
+
+  optind = 0;
+  while ((opt = getopt_long(argc, (char **)tmp_argv, "ir", options, NULL)) != -1) {
+    switch (opt) {
+      case 'i':
+        instance = 1;
+        break;
+      case 'r':
+        related = 1;
+        break;
+      default:
+        owl_function_makemsg("Wrong number of arguments for %s (%c)", argv[0], opt);
+        goto done;
+    }
   }
+
+  filtname = owl_function_smartfilter(instance, related);
+
   if (filtname) {
     owl_function_change_currentview_filter(filtname);
     owl_free(filtname);
   }
+
+done:
+  owl_free(tmp_argv);
+
   return NULL;
 }
 
@@ -1303,9 +1337,9 @@ char *owl_command_smartfilter(int argc, const char *const *argv, const char *buf
   char *filtname = NULL;
 
   if (argc == 1) {
-    filtname = owl_function_smartfilter(0);
+    filtname = owl_function_smartfilter(0, 0);
   } else if (argc == 2 && (!strcmp(argv[1], "-i") || !strcmp(argv[1], "--instance"))) {
-    filtname = owl_function_smartfilter(1);
+    filtname = owl_function_smartfilter(1, 0);
   } else {
     owl_function_makemsg("Wrong number of arguments for %s", argv[0]);    
   }
@@ -1631,7 +1665,7 @@ char *owl_command_multi(int argc, const char *const *argv, const char *buff)
     lastrv = owl_function_command(commands[i]);
   }
   owl_free(newbuff);
-  atokenize_free(commands, ncommands);
+  atokenize_delete(commands, ncommands);
   return lastrv;
 }
 
@@ -1672,6 +1706,34 @@ char *owl_command_bindkey(int argc, const char *const *argv, const char *buff)
   return NULL;
 }
 
+
+char *owl_command_unbindkey(int argc, const char *const *argv, const char *buf)
+{
+  owl_keymap *km;
+  int ret;
+
+  if (argc < 3) {
+    owl_function_makemsg("Usage: bindkey <keymap> <binding>");
+    return NULL;
+  }
+  km = owl_keyhandler_get_keymap(owl_global_get_keyhandler(&g), argv[1]);
+  if (!km) {
+    owl_function_makemsg("No such keymap '%s'", argv[1]);
+    return NULL;
+  }
+  ret = owl_keymap_remove_binding(km, argv[2]);
+  if (ret == -1) {
+    owl_function_makemsg("Unable to unbind '%s' in keymap '%s'.",
+			 argv[2], argv[1]);
+    return NULL;
+  } else if (ret == -2) {
+    owl_function_makemsg("No such binding '%s' in keymap '%s'.",
+                         argv[2], argv[1]);
+  }
+  return NULL;
+}
+
+
 void owl_command_quit(void)
 {
   owl_function_quit();
@@ -1861,10 +1923,10 @@ char *owl_command_zwrite(int argc, const char *const *argv, const char *buff)
   owl_zwrite_create_from_line(&z, buff);
   if (owl_zwrite_is_message_set(&z)) {
     owl_function_zwrite(buff, NULL);
-    owl_zwrite_free(&z);
+    owl_zwrite_cleanup(&z);
     return (NULL);
   }
-  owl_zwrite_free(&z);
+  owl_zwrite_cleanup(&z);
 
   if (argc < 2) {
     owl_function_makemsg("Not enough arguments to the zwrite command.");
@@ -1961,7 +2023,6 @@ char *owl_command_loopwrite(int argc, const char *const *argv, const char *buff)
 
 char *owl_command_zcrypt(int argc, const char *const *argv, const char *buff)
 {
-#ifdef OWL_ENABLE_ZCRYPT
   owl_zwrite z;
 
   if (!owl_global_is_havezephyr(&g)) {
@@ -1972,10 +2033,10 @@ char *owl_command_zcrypt(int argc, const char *const *argv, const char *buff)
   owl_zwrite_create_from_line(&z, buff);
   if (owl_zwrite_is_message_set(&z)) {
     owl_function_zcrypt(buff, NULL);
-    owl_zwrite_free(&z);
+    owl_zwrite_cleanup(&z);
     return (NULL);
   }
-  owl_zwrite_free(&z);
+  owl_zwrite_cleanup(&z);
 
   if (argc < 2) {
     owl_function_makemsg("Not enough arguments to the zcrypt command.");
@@ -1983,10 +2044,6 @@ char *owl_command_zcrypt(int argc, const char *const *argv, const char *buff)
     owl_function_zwrite_setup(buff);
   }
   return(NULL);
-#else
-  owl_function_makemsg("This Owl does not support zcrypt");
-  return NULL;
-#endif
 }
 
 char *owl_command_reply(int argc, const char *const *argv, const char *buff)
@@ -2126,10 +2183,6 @@ char *owl_command_view(int argc, const char *const *argv, const char *buff)
       owl_function_change_currentview_filter(owl_global_get_view_home(&g));
       argc--;
       argv++;
-    } else if (!strcmp(argv[0], "-r")) {
-      const char *foo;
-      foo=owl_function_create_negative_filter(owl_view_get_filtname(owl_global_get_current_view(&g)));
-      owl_function_change_currentview_filter(foo);
     } else if (!strcmp(argv[0], "-s")) {
       if (argc<2) {
 	owl_function_makemsg("Too few argments to the view command");
@@ -2224,7 +2277,7 @@ char *owl_command_viewclass(int argc, const char *const *argv, const char *buff)
     owl_function_makemsg("Wrong number of arguments to viewclass command");
     return NULL;
   }
-  filtname = owl_function_classinstfilt(argv[1], NULL);
+  filtname = owl_function_classinstfilt(argv[1], NULL, owl_global_is_narrow_related(&g));
   owl_function_change_currentview_filter(filtname);
   owl_free(filtname);
   return NULL;
@@ -2336,7 +2389,7 @@ char *owl_command_colorclass(int argc, const char *const *argv, const char *buff
     return NULL;
   }
 
-  filtname=owl_function_classinstfilt(argv[1], NULL);
+  filtname=owl_function_classinstfilt(argv[1], NULL, owl_global_is_narrow_related(&g));
   (void) owl_function_color_filter(filtname, argv[2], (argc == 4 ? argv[3] : NULL));
   return NULL;
 }
@@ -2528,9 +2581,9 @@ char *owl_command_aimlogin(int argc, const char *const *argv, const char *buff)
 
   /* if we get two arguments, ask for the password */
   if (argc==2) {
-    owl_global_set_buffercommand(&g, argv[1]);
-    owl_global_set_buffercallback(&g, &owl_callback_aimlogin);
-    owl_function_start_password("AIM Password: ");
+    owl_editwin *e = owl_function_start_password("AIM Password: ");
+    owl_editwin_set_cbdata(e, owl_strdup(argv[1]), owl_free);
+    owl_editwin_set_callback(e, owl_callback_aimlogin);
     return(NULL);
   } else {
     owl_function_aimlogin(argv[1], argv[2]);
@@ -2659,13 +2712,11 @@ void owl_command_edit_cancel(owl_editwin *e)
     owl_history_reset(hist);
   }
 
-  owl_editwin_fullclear(e);
   owl_global_set_needrefresh(&g);
-  wnoutrefresh(owl_editwin_get_curswin(e));
-  owl_global_set_typwin_inactive(&g);
-  owl_editwin_new_style(e, OWL_EDITWIN_STYLE_ONELINE, NULL);
+  owl_global_pop_context(&g);
 
-  owl_function_activate_keymap("recv");
+  owl_global_set_typwin_inactive(&g);
+  owl_editwin_delete(e);
 }
 
 void owl_command_edit_history_prev(owl_editwin *e)
@@ -2682,7 +2733,7 @@ void owl_command_edit_history_prev(owl_editwin *e)
   if (ptr) {
     owl_editwin_clear(e);
     owl_editwin_insert_string(e, ptr);
-    owl_editwin_redisplay(e, 0);
+    owl_editwin_redisplay(e);
     owl_global_set_needrefresh(&g);
   } else {
     owl_function_beep();
@@ -2699,7 +2750,7 @@ void owl_command_edit_history_next(owl_editwin *e)
   if (ptr) {
     owl_editwin_clear(e);
     owl_editwin_insert_string(e, ptr);
-    owl_editwin_redisplay(e, 0);
+    owl_editwin_redisplay(e);
     owl_global_set_needrefresh(&g);
   } else {
     owl_function_beep();
@@ -2710,58 +2761,26 @@ char *owl_command_edit_insert_text(owl_editwin *e, int argc, const char *const *
 {
   buff = skiptokens(buff, 1);
   owl_editwin_insert_string(e, buff);
-  owl_editwin_redisplay(e, 0);
+  owl_editwin_redisplay(e);
   owl_global_set_needrefresh(&g);  
   return NULL;
 }
 
-void owl_command_editline_done(owl_editwin *e)
-{
-  owl_history *hist=owl_editwin_get_history(e);
-  char *rv, *cmd;
-
-  owl_history_store(hist, owl_editwin_get_text(e));
-  owl_history_reset(hist);
-  owl_global_set_typwin_inactive(&g);
-  cmd = owl_strdup(owl_editwin_get_text(e));
-  owl_editwin_fullclear(e);
-  rv = owl_function_command(cmd);
-  owl_free(cmd);
-
-  wnoutrefresh(owl_editwin_get_curswin(e));
-  owl_global_set_needrefresh(&g);
-
-  if (rv) {
-    owl_function_makemsg("%s", rv);
-    owl_free(rv);
-  }
-}
-
-
-void owl_command_editresponse_done(owl_editwin *e)
-{
-  owl_function_run_buffercommand();
-
-  owl_global_set_typwin_inactive(&g);
-  owl_editwin_fullclear(e);
-  wnoutrefresh(owl_editwin_get_curswin(e));
-  owl_global_set_needrefresh(&g);
-}
-
-
 void owl_command_edit_done(owl_editwin *e)
 {
   owl_history *hist=owl_editwin_get_history(e);
 
-  owl_history_store(hist, owl_editwin_get_text(e));
-  owl_history_reset(hist);
+  if (hist) {
+    owl_history_store(hist, owl_editwin_get_text(e));
+    owl_history_reset(hist);
+  }
 
-  owl_function_run_buffercommand();
-  owl_editwin_new_style(e, OWL_EDITWIN_STYLE_ONELINE, NULL);
-  owl_editwin_fullclear(e);
   owl_global_set_typwin_inactive(&g);
+  owl_global_pop_context(&g);
   owl_global_set_needrefresh(&g);
-  wnoutrefresh(owl_editwin_get_curswin(e));
+
+  owl_editwin_do_callback(e);
+  owl_editwin_delete(e);
 }
 
 void owl_command_edit_done_or_delete(owl_editwin *e)
@@ -2781,6 +2800,7 @@ void owl_command_edit_done_or_delete(owl_editwin *e)
 void owl_command_popless_quit(owl_viewwin *vw)
 {
   owl_popwin_close(owl_global_get_popwin(&g));
-  owl_viewwin_free(vw);
+  owl_global_pop_context(&g);
+  owl_viewwin_cleanup(vw);
   owl_global_set_needrefresh(&g);
 }
