@@ -34,8 +34,8 @@ struct _owl_editwin { /*noproto*/
   int echochar;
   oe_excursion *excursions;
 
-  char *command;
   void (*callback)(struct _owl_editwin*);
+  void (*destroy_cbdata)(void *);
   void *cbdata;
 };
 
@@ -44,7 +44,6 @@ static void oe_save_excursion(owl_editwin *e, oe_excursion *x);
 static void oe_release_excursion(owl_editwin *e, oe_excursion *x);
 static void oe_restore_excursion(owl_editwin *e, oe_excursion *x);
 static void oe_restore_mark_only(owl_editwin *e, oe_excursion *x);
-static int oe_count_glyphs(const char *s);
 static int oe_char_width(gunichar c, int column);
 static int oe_region_width(owl_editwin *e, int start, int end, int width);
 static int oe_find_display_line(owl_editwin *e, int *x, int index);
@@ -57,12 +56,13 @@ static int owl_editwin_replace_internal(owl_editwin *e, int replace, const char
 static const char *oe_copy_buf(owl_editwin *e, const char *buf, int len);
 static int oe_copy_region(owl_editwin *e);
 static char *oe_chunk(owl_editwin *e, int start, int end);
+static void oe_destroy_cbdata(owl_editwin *e);
 
 #define INCR 4096
 
 #define WHITESPACE " \n\t"
 
-owl_editwin *owl_editwin_allocate(void)
+static owl_editwin *owl_editwin_allocate(void)
 {
   owl_editwin *e;
   e = owl_malloc(sizeof(owl_editwin));
@@ -70,31 +70,19 @@ owl_editwin *owl_editwin_allocate(void)
   return e;
 }
 
-void owl_editwin_free(owl_editwin *e)
+void owl_editwin_delete(owl_editwin *e)
 {
   owl_free(e->buff);
   owl_free(e->killbuf);
-  owl_free(e->command);
   /* just in case someone forgot to clean up */
   while (e->excursions) {
     oe_release_excursion(e, e->excursions);
   }
+  oe_destroy_cbdata(e);
 
   owl_free(e);
 }
 
-static int oe_count_glyphs(const char *s)
-{
-  int count = 0;
-  const char *p;
-
-  for(p = s; *p != 0; p = g_utf8_find_next_char(p, NULL))
-    if (!g_unichar_ismark(g_utf8_get_char(p)))
-      count++;
-
-  return count;
-}
-
 static inline void oe_set_index(owl_editwin *e, int index)
 {
   if (index != e->index) {
@@ -115,10 +103,12 @@ void owl_editwin_set_mark(owl_editwin *e)
   /* owl_function_makemsg("Mark set."); */
 }
 
-/* initialize the editwin e.
- * 'win' is an already initialzed curses window that will be used by editwin
- */
-void owl_editwin_init(owl_editwin *e, WINDOW *win, int winlines, int wincols, int style, owl_history *hist)
+static void _owl_editwin_init(owl_editwin *e,
+                              WINDOW *win,
+                              int winlines,
+                              int wincols,
+                              int style,
+                              owl_history *hist)
 {
   e->buff=owl_malloc(INCR);
   e->buff[0]='\0';
@@ -134,33 +124,37 @@ void owl_editwin_init(owl_editwin *e, WINDOW *win, int winlines, int wincols, in
   e->cursorx = -1;
   e->topindex = 0;
   e->excursions = NULL;
-  owl_editwin_set_curswin(e, win, winlines, wincols);
   e->style=style;
   if ((style!=OWL_EDITWIN_STYLE_MULTILINE) &&
       (style!=OWL_EDITWIN_STYLE_ONELINE)) {
     e->style=OWL_EDITWIN_STYLE_MULTILINE;
   }
+  owl_editwin_set_curswin(e, win, winlines, wincols);
   e->lock=0;
   e->dotsend=0;
   e->echochar='\0';
 
-  /* We get initialized multiple times, but we need to hold on to
-     the callbacks, so we can't NULL them here. */
-  /*
-    e->command = NULL;
-    e->callback = NULL;
-    e->cbdata = NULL;
-  */
   if (win) werase(win);
 }
 
+owl_editwin *owl_editwin_new(WINDOW *win, int winlines, int wincols, int style, owl_history *hist)
+{
+  owl_editwin *e = owl_editwin_allocate();
+
+  _owl_editwin_init(e, win, winlines, wincols, style, hist);
+  return e;
+}
+
 void owl_editwin_set_curswin(owl_editwin *e, WINDOW *w, int winlines, int wincols)
 {
   e->curswin=w;
   e->winlines=winlines;
   e->wincols=wincols;
   e->fillcol=owl_editwin_limit_maxcols(wincols-7, owl_global_get_edit_maxfillcols(&g));
-  e->wrapcol=owl_editwin_limit_maxcols(wincols-7, owl_global_get_edit_maxwrapcols(&g));
+  if (e->style == OWL_EDITWIN_STYLE_MULTILINE)
+    e->wrapcol=owl_editwin_limit_maxcols(wincols-7, owl_global_get_edit_maxwrapcols(&g));
+  else
+    e->wrapcol = 0;
 }
 
 /* echo the character 'ch' for each normal character keystroke,
@@ -187,18 +181,6 @@ void owl_editwin_set_dotsend(owl_editwin *e)
   e->dotsend=1;
 }
 
-void owl_editwin_set_command(owl_editwin *e, const char *command)
-{
-  if(e->command) owl_free(e->command);
-  e->command = owl_strdup(command);
-}
-
-const char *owl_editwin_get_command(owl_editwin *e)
-{
-  if(e->command) return e->command;
-  return "";
-}
-
 void owl_editwin_set_callback(owl_editwin *e, void (*cb)(owl_editwin*))
 {
   e->callback = cb;
@@ -209,9 +191,18 @@ void (*owl_editwin_get_callback(owl_editwin *e))(owl_editwin*)
   return e->callback;
 }
 
-void owl_editwin_set_cbdata(owl_editwin *e, void *data)
+static void oe_destroy_cbdata(owl_editwin *e) {
+  if (e->destroy_cbdata)
+    e->destroy_cbdata(e->cbdata);
+  e->cbdata = NULL;
+  e->destroy_cbdata = NULL;
+}
+
+void owl_editwin_set_cbdata(owl_editwin *e, void *data, void (*destroy)(void *))
 {
+  oe_destroy_cbdata(e);
   e->cbdata = data;
+  e->destroy_cbdata = destroy;
 }
 
 void *owl_editwin_get_cbdata(owl_editwin *e) {
@@ -250,7 +241,7 @@ void owl_editwin_set_locktext(owl_editwin *e, const char *text)
   e->buff[e->bufflen] = 0;
   e->lock=e->bufflen;
   oe_set_index(e, e->lock);
-  owl_editwin_redisplay(e, 0);
+  owl_editwin_redisplay(e);
 }
 
 int owl_editwin_get_style(owl_editwin *e)
@@ -258,31 +249,6 @@ int owl_editwin_get_style(owl_editwin *e)
   return(e->style);
 }
 
-void owl_editwin_new_style(owl_editwin *e, int newstyle, owl_history *h)
-{
-  e->hist = h;
-
-  if (e->style==newstyle) return;
-
-  if (newstyle==OWL_EDITWIN_STYLE_MULTILINE) {
-    e->style=newstyle;
-  } else if (newstyle==OWL_EDITWIN_STYLE_ONELINE) {
-    e->style=newstyle;
-
-    /* nuke everything after the first line */
-    owl_editwin_move_to_top(e);
-    owl_editwin_move_to_end_of_line(e);
-    owl_editwin_replace(e, oe_count_glyphs(e->buff + e->index),  "");
-  }
-}
-
-/* completly reinitialize the buffer */
-void owl_editwin_fullclear(owl_editwin *e)
-{
-  owl_free(e->buff);
-  owl_editwin_init(e, e->curswin, e->winlines, e->wincols, e->style, e->hist);
-}
-
 /* clear all text except for locktext and put the cursor at the
  * beginning
  */
@@ -301,7 +267,7 @@ void owl_editwin_clear(owl_editwin *e)
   }
 
   owl_free(e->buff);
-  owl_editwin_init(e, e->curswin, e->winlines, e->wincols, e->style, e->hist);
+  _owl_editwin_init(e, e->curswin, e->winlines, e->wincols, e->style, e->hist);
 
   if (lock > 0) {
     owl_editwin_set_locktext(e, locktext);
@@ -517,8 +483,7 @@ static void oe_mvaddnec(owl_editwin *e, int y, int x, int count)
 }
 
 /* regenerate the text on the curses window */
-/* if update == 1 then do a doupdate(), otherwise do not */
-void owl_editwin_redisplay(owl_editwin *e, int update)
+void owl_editwin_redisplay(owl_editwin *e)
 {
   int x = -1, y = -1, t;
   int line, index, lineindex, times = 0;
@@ -553,7 +518,7 @@ void owl_editwin_redisplay(owl_editwin *e, int update)
 			oe_region_width(e, e->lock, index,
 					oe_region_width(e, lineindex, e->lock, 0)));
 	  } else
-	    oe_mvaddnec(e, line, 0, oe_region_width(e, line, index, 0));
+	    oe_mvaddnec(e, line, 0, oe_region_width(e, lineindex, index, 0));
 	}
       }
       line++;
@@ -565,10 +530,6 @@ void owl_editwin_redisplay(owl_editwin *e, int update)
 
   wmove(e->curswin, y, x);
   e->cursorx = x;
-
-  wnoutrefresh(e->curswin);
-  if (update == 1)
-    doupdate();
 }
 
 static inline void oe_fixup(int *target, int start, int end, int change) {
@@ -584,6 +545,12 @@ int owl_editwin_replace_region(owl_editwin *e, const char *s)
 {
   oe_excursion x;
   int ret;
+
+  if (e->mark == -1) {
+    owl_function_error("The mark is unset, there is no region to replace.");
+    return 0;
+  }
+
   oe_save_excursion(e, &x);
 
   if(e->index > e->mark) {
@@ -1237,7 +1204,7 @@ void owl_editwin_post_process_char(owl_editwin *e, owl_input j)
     owl_command_edit_done(e);
     return;
   }
-  owl_editwin_redisplay(e, 0);
+  owl_editwin_redisplay(e);
 }
 
 static int oe_region_width(owl_editwin *e, int start, int end, int offset)
@@ -1267,7 +1234,8 @@ static void oe_insert_char(owl_editwin *e, gunichar c)
       return;
     }
 
-    if (e->cursorx != -1 && e->cursorx + oe_char_width(c, e->cursorx) > e->wrapcol) {
+    if (e->wrapcol > 0 && e->cursorx != -1 &&
+        e->cursorx + oe_char_width(c, e->cursorx) > e->wrapcol) {
       /* XXX this is actually wrong:
        * + If the line has been been wrapped, we can be past the wrap column but
        *   e->cursorx be much smaller.
