@@ -11,6 +11,7 @@
 #include <errno.h>
 #include <signal.h>
 #include "owl.h"
+#include "filterproc.h"
 
 char *owl_function_command(const char *cmdbuff)
 {
@@ -55,8 +56,8 @@ void owl_function_show_commands(void)
   owl_fmtext_append_list(&fm, &l, "\n", owl_function_cmd_describe);
   owl_fmtext_append_normal(&fm, "\n");
   owl_function_popless_fmtext(&fm);
-  owl_cmddict_namelist_free(&l);
-  owl_fmtext_free(&fm);
+  owl_cmddict_namelist_cleanup(&l);
+  owl_fmtext_cleanup(&fm);
 }
 
 void owl_function_show_view(const char *viewname)
@@ -74,7 +75,7 @@ void owl_function_show_view(const char *viewname)
   owl_fmtext_init_null(&fm);
   owl_view_to_fmtext(v, &fm);
   owl_function_popless_fmtext(&fm);
-  owl_fmtext_free(&fm);
+  owl_fmtext_cleanup(&fm);
 }
 
 void owl_function_show_styles(void) {
@@ -87,8 +88,8 @@ void owl_function_show_styles(void) {
   owl_fmtext_append_list(&fm, &l, "\n", owl_function_style_describe);
   owl_fmtext_append_normal(&fm, "\n");
   owl_function_popless_fmtext(&fm);
-  owl_list_free_all(&l, owl_free);
-  owl_fmtext_free(&fm);
+  owl_list_cleanup(&l, owl_free);
+  owl_fmtext_cleanup(&fm);
 }
 
 char *owl_function_style_describe(const char *name) {
@@ -125,7 +126,7 @@ void owl_function_show_license(void)
 
   text=""
     "barnowl version " OWL_VERSION_STRING "\n"
-    "Copyright (c) 2006-2009 The BarnOwl Developers. All rights reserved.\n"
+    "Copyright (c) 2006-2010 The BarnOwl Developers. All rights reserved.\n"
     "Copyright (c) 2004 James Kretchmar. All rights reserved.\n"
     "\n"
     "Redistribution and use in source and binary forms, with or without\n"
@@ -214,10 +215,6 @@ void owl_function_adminmsg(const char *header, const char *body)
 
   /* redisplay etc. */
   owl_mainwin_redisplay(owl_global_get_mainwin(&g));
-  if (owl_popwin_is_active(owl_global_get_popwin(&g))) {
-    owl_popwin_refresh(owl_global_get_popwin(&g));
-  }
-  wnoutrefresh(owl_global_get_curs_recwin(&g));
   owl_global_set_needrefresh(&g);
 }
 
@@ -237,7 +234,7 @@ owl_message *owl_function_make_outgoing_zephyr(const char *body, const char *zwr
   m=owl_malloc(sizeof(owl_message));
   
   owl_message_create_from_zwrite(m, &zw, body);
-  owl_zwrite_free(&zw);
+  owl_zwrite_cleanup(&zw);
 
   return(m);
 }
@@ -280,26 +277,22 @@ owl_message *owl_function_make_outgoing_loopback(const char *body)
   return(m);
 }
 
-void owl_function_start_edit_win(const char *line, void (*callback)(owl_editwin *), void *data)
+void owl_function_start_edit_win(const char *line, void (*callback)(owl_editwin *), void *data, void (*cleanup)(void *))
 {
   owl_editwin *e;
   char *s;
 
   /* create and setup the editwin */
-  e = owl_global_get_typwin(&g);
-  owl_editwin_new_style(e, OWL_EDITWIN_STYLE_MULTILINE,
-			owl_global_get_msg_history(&g));
-  owl_editwin_clear(e);
+  e = owl_global_set_typwin_active(&g, OWL_EDITWIN_STYLE_MULTILINE,
+                                   owl_global_get_msg_history(&g));
   owl_editwin_set_dotsend(e);
   s = owl_sprintf("----> %s\n", line);
   owl_editwin_set_locktext(e, s);
   owl_free(s);
 
-  /* make it active */
-  owl_global_set_typwin_active(&g);
-
-  owl_editwin_set_cbdata(owl_global_get_typwin(&g), data);
-  owl_global_set_buffercallback(&g, callback);
+  owl_editwin_set_cbdata(e, data, cleanup);
+  owl_editwin_set_callback(e, callback);
+  owl_global_push_context(&g, OWL_CTX_EDITMULTI, e, "editmulti");
 }
 
 static void owl_function_write_setup(const char *line, const char *noun, void (*callback)(owl_editwin *))
@@ -314,8 +307,9 @@ static void owl_function_write_setup(const char *line, const char *noun, void (*
 			 "End with a dot on a line by itself.  ^C will quit.",
 			 noun);
 
-  owl_function_start_edit_win(line, callback, NULL);
-  owl_global_set_buffercommand(&g, line);
+  owl_function_start_edit_win(line, callback,
+                              owl_strdup(line),
+                              owl_free);
 }
 
 void owl_function_zwrite_setup(const char *line)
@@ -327,7 +321,7 @@ void owl_function_zwrite_setup(const char *line)
   ret=owl_zwrite_create_from_line(&z, line);
   if (ret) {
     owl_function_error("Error in zwrite arguments");
-    owl_zwrite_free(&z);
+    owl_zwrite_cleanup(&z);
     return;
   }
 
@@ -335,7 +329,7 @@ void owl_function_zwrite_setup(const char *line)
   if (owl_global_is_txping(&g)) {
     owl_zwrite_send_ping(&z);
   }
-  owl_zwrite_free(&z);
+  owl_zwrite_cleanup(&z);
 
   owl_function_write_setup(line, "zephyr", &owl_callback_zwrite);
 }
@@ -347,31 +341,12 @@ void owl_function_aimwrite_setup(const char *line)
 
 void owl_function_loopwrite_setup(void)
 {
-  owl_editwin *e;
-
-  /* create and setup the editwin */
-  e=owl_global_get_typwin(&g);
-  owl_editwin_new_style(e, OWL_EDITWIN_STYLE_MULTILINE, owl_global_get_msg_history(&g));
-
-  if (!owl_global_get_lockout_ctrld(&g)) {
-    owl_function_makemsg("Type your message below.  End with ^D or a dot on a line by itself.  ^C will quit.");
-  } else {
-    owl_function_makemsg("Type your message below.  End with a dot on a line by itself.  ^C will quit.");
-  }
-
-  owl_editwin_clear(e);
-  owl_editwin_set_dotsend(e);
-  owl_editwin_set_locktext(e, "----> loopwrite\n");
-
-  /* make it active */
-  owl_global_set_typwin_active(&g);
-
-  owl_global_set_buffercommand(&g, "loopwrite");
-  owl_global_set_buffercallback(&g, &owl_callback_loopwrite);
+  owl_function_write_setup("loopwrite", "message", owl_callback_loopwrite);
 }
 
 void owl_callback_zwrite(owl_editwin *e) {
-  owl_function_zwrite(owl_editwin_get_command(e),
+  char *command = owl_editwin_get_cbdata(e);
+  owl_function_zwrite(command,
                       owl_editwin_get_text(e));
 }
 
@@ -412,7 +387,7 @@ void owl_function_zwrite(const char *line, const char *msg)
   }
 
   /* free the zwrite */
-  owl_zwrite_free(&z);
+  owl_zwrite_cleanup(&z);
 }
 
 /* send, log and display an outgoing zcrypt zephyr.  If 'msg' is NULL
@@ -424,6 +399,9 @@ void owl_function_zcrypt(const char *line, const char *msg)
   const char *mymsg;
   char *cryptmsg;
   owl_message *m;
+  const char *argv[7];
+  char *zcrypt;
+  int rv, status;
 
   /* create the zwrite and send the message */
   owl_zwrite_create_from_line(&z, line);
@@ -433,17 +411,25 @@ void owl_function_zcrypt(const char *line, const char *msg)
   }
 
   mymsg=owl_zwrite_get_message(&z);
-#ifdef OWL_ENABLE_ZCRYPT
-  cryptmsg = owl_zcrypt_encrypt(mymsg, owl_zwrite_get_class(&z), owl_zwrite_get_instance(&z));
-  if (!cryptmsg) {
+
+  zcrypt = owl_sprintf("%s/zcrypt", owl_get_bindir());
+  argv[0] = "zcrypt";
+  argv[1] = "-E";
+  argv[2] = "-c"; argv[3] = owl_zwrite_get_class(&z);
+  argv[4] = "-i"; argv[5] = owl_zwrite_get_instance(&z);
+  argv[6] = NULL;
+
+  rv = call_filter(zcrypt, argv, mymsg, &cryptmsg, &status);
+
+  owl_free(zcrypt);
+
+  if (rv || status) {
+    if(cryptmsg) owl_free(cryptmsg);
     owl_function_error("Error in zcrypt, possibly no key found.  Message not sent.");
     owl_function_beep();
-    owl_zwrite_free(&z);
+    owl_zwrite_cleanup(&z);
     return;
   }
-#else
-  cryptmsg=owl_strdup(mymsg);
-#endif
 
   owl_zwrite_set_message(&z, cryptmsg);
   owl_zwrite_set_opcode(&z, "crypt");
@@ -465,11 +451,12 @@ void owl_function_zcrypt(const char *line, const char *msg)
 
   /* free the zwrite */
   owl_free(cryptmsg);
-  owl_zwrite_free(&z);
+  owl_zwrite_cleanup(&z);
 }
 
 void owl_callback_aimwrite(owl_editwin *e) {
-  owl_function_aimwrite(owl_editwin_get_command(e),
+  char *command = owl_editwin_get_cbdata(e);
+  owl_function_aimwrite(command,
                         owl_editwin_get_text(e));
 }
 
@@ -904,7 +891,8 @@ void owl_function_loadloginsubs(const char *file)
 }
 
 void owl_callback_aimlogin(owl_editwin *e) {
-  owl_function_aimlogin(owl_editwin_get_command(e),
+  char *user = owl_editwin_get_cbdata(e);
+  owl_function_aimlogin(user,
                         owl_editwin_get_text(e));
 }
 
@@ -1205,11 +1193,6 @@ void owl_function_resize(void)
   owl_global_set_resize_pending(&g);
 }
 
-void owl_function_run_buffercommand(void)
-{
-  owl_editwin_do_callback(owl_global_get_typwin(&g));
-}
-
 void owl_function_debugmsg(const char *fmt, ...)
 {
   FILE *file;
@@ -1220,7 +1203,7 @@ void owl_function_debugmsg(const char *fmt, ...)
   if (!owl_global_is_debug_fast(&g))
     return;
 
-  file = fopen(owl_global_get_debug_file(&g), "a");
+  file = owl_global_get_debug_file_handle(&g);
   if (!file) /* XXX should report this */
     return;
 
@@ -1230,7 +1213,7 @@ void owl_function_debugmsg(const char *fmt, ...)
 	  (int) getpid(), ctime(&now), now - owl_global_get_starttime(&g));
   vfprintf(file, fmt, ap);
   putc('\n', file);
-  fclose(file);
+  fflush(file);
 
   va_end(ap);
 }
@@ -1270,6 +1253,11 @@ void owl_function_unsubscribe(const char *class, const char *inst, const char *r
 
 void owl_function_set_cursor(WINDOW *win)
 {
+  /* Be careful that this window is actually empty, otherwise panels get confused */
+  if (is_wintouched(win)) {
+    owl_function_debugmsg("Warning: owl_function_set_cursor called on dirty window");
+    update_panels();
+  }
   wnoutrefresh(win);
 }
 
@@ -1283,15 +1271,6 @@ void owl_function_full_redisplay(void)
   if (g.lines >= 2)
       redrawwin(owl_global_get_curs_msgwin(&g));
 
-  wnoutrefresh(owl_global_get_curs_recwin(&g));
-  wnoutrefresh(owl_global_get_curs_sepwin(&g));
-  wnoutrefresh(owl_global_get_curs_typwin(&g));
-  wnoutrefresh(owl_global_get_curs_msgwin(&g));
-
-  if (owl_popwin_is_active(owl_global_get_popwin(&g))) {
-    owl_popwin_refresh(owl_global_get_popwin(&g));
-  }
-  
   sepbar("");
   owl_function_makemsg("");
 
@@ -1307,11 +1286,11 @@ void owl_function_popless_text(const char *text)
   v=owl_global_get_viewwin(&g);
 
   owl_popwin_up(pw);
+  owl_global_push_context(&g, OWL_CTX_POPLESS, v, "popless");
   owl_viewwin_init_text(v, owl_popwin_get_curswin(pw),
 			owl_popwin_get_lines(pw), owl_popwin_get_cols(pw),
 			text);
-  owl_popwin_refresh(pw);
-  owl_viewwin_redisplay(v, 0);
+  owl_viewwin_redisplay(v);
   owl_global_set_needrefresh(&g);
 }
 
@@ -1324,11 +1303,11 @@ void owl_function_popless_fmtext(const owl_fmtext *fm)
   v=owl_global_get_viewwin(&g);
 
   owl_popwin_up(pw);
+  owl_global_push_context(&g, OWL_CTX_POPLESS, v, "popless");
   owl_viewwin_init_fmtext(v, owl_popwin_get_curswin(pw),
 		   owl_popwin_get_lines(pw), owl_popwin_get_cols(pw),
 		   fm);
-  owl_popwin_refresh(pw);
-  owl_viewwin_redisplay(v, 0);
+  owl_viewwin_redisplay(v);
   owl_global_set_needrefresh(&g);
 }
 
@@ -1350,7 +1329,7 @@ void owl_function_popless_file(const char *filename)
   owl_free(s);
 
   owl_function_popless_fmtext(&fm);
-  owl_fmtext_free(&fm);
+  owl_fmtext_cleanup(&fm);
   fclose(file);
 }
 
@@ -1370,7 +1349,7 @@ void owl_function_about(void)
     "between Witches and Wizards. The name 'barnowl' was chosen\n"
     "because we feel our owls should live closer to our ponies.\n"
     "\n"
-    "Copyright (c) 2006-2009 The BarnOwl Developers. All rights reserved.\n"
+    "Copyright (c) 2006-2010 The BarnOwl Developers. All rights reserved.\n"
     "Copyright (c) 2004 James Kretchmar. All rights reserved.\n"
     "Copyright 2002 Massachusetts Institute of Technology\n"
     "\n"
@@ -1507,8 +1486,8 @@ void owl_function_info(void)
   owl_fmtext_append_fmtext(&fm, &attrfm);
   
   owl_function_popless_fmtext(&fm);
-  owl_fmtext_free(&fm);
-  owl_fmtext_free(&attrfm);
+  owl_fmtext_cleanup(&fm);
+  owl_fmtext_cleanup(&attrfm);
 }
 
 /* print the current message in a popup window.
@@ -1536,7 +1515,7 @@ void owl_function_curmsg_to_popwin(void)
   owl_style_get_formattext(s, &fm, m);
 
   owl_function_popless_fmtext(&fm);
-  owl_fmtext_free(&fm);
+  owl_fmtext_cleanup(&fm);
 }
 
 void owl_function_page_curmsg(int step)
@@ -1655,7 +1634,7 @@ void owl_function_printallvars(void)
     }
   }
   g_string_append(str, "\n");
-  owl_variable_dict_namelist_free(&varnames);
+  owl_variable_dict_namelist_cleanup(&varnames);
 
   owl_function_popless_text(str->str);
   g_string_free(str, TRUE);
@@ -1679,9 +1658,9 @@ void owl_function_show_variables(void)
       owl_variable_describe(owl_global_get_vardict(&g), varname, &fm);
     }
   }
-  owl_variable_dict_namelist_free(&varnames);
+  owl_variable_dict_namelist_cleanup(&varnames);
   owl_function_popless_fmtext(&fm);
-  owl_fmtext_free(&fm);
+  owl_fmtext_cleanup(&fm);
 }
 
 void owl_function_show_variable(const char *name)
@@ -1691,7 +1670,7 @@ void owl_function_show_variable(const char *name)
   owl_fmtext_init_null(&fm);
   owl_variable_get_help(owl_global_get_vardict(&g), name, &fm);
   owl_function_popless_fmtext(&fm);
-  owl_fmtext_free(&fm);  
+  owl_fmtext_cleanup(&fm);
 }
 
 /* note: this applies to global message list, not to view.
@@ -1825,7 +1804,7 @@ void owl_function_status(void)
   }
 
   owl_function_popless_fmtext(&fm);
-  owl_fmtext_free(&fm);
+  owl_fmtext_cleanup(&fm);
 }
 
 void owl_function_show_term(void)
@@ -1846,7 +1825,7 @@ void owl_function_show_term(void)
   }
 
   owl_function_popless_fmtext(&fm);
-  owl_fmtext_free(&fm);
+  owl_fmtext_cleanup(&fm);
 }
 
 /* if type = 0 then normal reply.
@@ -1928,61 +1907,67 @@ void owl_function_zlocate(int argc, const char *const *argv, int auth)
   }
 
   owl_function_popless_fmtext(&fm);
-  owl_fmtext_free(&fm);
+  owl_fmtext_cleanup(&fm);
+}
+
+void owl_callback_command(owl_editwin *e)
+{
+  char *rv;
+  const char *line = owl_editwin_get_text(e);
+
+  rv = owl_function_command(line);
+   if (rv) {
+    owl_function_makemsg("%s", rv);
+    owl_free(rv);
+  }
 }
 
 void owl_function_start_command(const char *line)
 {
   owl_editwin *tw;
 
-  tw=owl_global_get_typwin(&g);
-  owl_global_set_typwin_active(&g);
-  owl_editwin_new_style(tw, OWL_EDITWIN_STYLE_ONELINE, 
-			owl_global_get_cmd_history(&g));
+  tw = owl_global_set_typwin_active(&g, OWL_EDITWIN_STYLE_ONELINE, owl_global_get_cmd_history(&g));
 
   owl_editwin_set_locktext(tw, "command: ");
   owl_global_set_needrefresh(&g);
 
   owl_editwin_insert_string(tw, line);
-  owl_editwin_redisplay(tw, 0);
+  owl_editwin_redisplay(tw);
 
-  owl_context_set_editline(owl_global_get_context(&g), tw);
-  owl_function_activate_keymap("editline");
+  owl_global_push_context(&g, OWL_CTX_EDITLINE, tw, "editline");
+  owl_editwin_set_callback(tw, owl_callback_command);
 }
 
-void owl_function_start_question(const char *line)
+owl_editwin *owl_function_start_question(const char *line)
 {
   owl_editwin *tw;
 
-  tw=owl_global_get_typwin(&g);
-  owl_global_set_typwin_active(&g);
-  owl_editwin_new_style(tw, OWL_EDITWIN_STYLE_ONELINE, owl_global_get_cmd_history(&g));
+  tw = owl_global_set_typwin_active(&g, OWL_EDITWIN_STYLE_ONELINE, owl_global_get_cmd_history(&g));
 
   owl_editwin_set_locktext(tw, line);
   owl_global_set_needrefresh(&g);
 
-  owl_editwin_redisplay(tw, 0);
+  owl_editwin_redisplay(tw);
 
-  owl_context_set_editresponse(owl_global_get_context(&g), tw);
-  owl_function_activate_keymap("editresponse");
+  owl_global_push_context(&g, OWL_CTX_EDITRESPONSE, tw, "editresponse");
+  return tw;
 }
 
-void owl_function_start_password(const char *line)
+owl_editwin *owl_function_start_password(const char *line)
 {
   owl_editwin *tw;
 
-  tw=owl_global_get_typwin(&g);
-  owl_global_set_typwin_active(&g);
-  owl_editwin_new_style(tw, OWL_EDITWIN_STYLE_ONELINE, owl_global_get_cmd_history(&g));
+  tw = owl_global_set_typwin_active(&g, OWL_EDITWIN_STYLE_ONELINE, NULL);
+
   owl_editwin_set_echochar(tw, '*');
 
   owl_editwin_set_locktext(tw, line);
   owl_global_set_needrefresh(&g);
 
-  owl_editwin_redisplay(tw, 0);
+  owl_editwin_redisplay(tw);
 
-  owl_context_set_editresponse(owl_global_get_context(&g), tw);
-  owl_function_activate_keymap("editresponse");
+  owl_global_push_context(&g, OWL_CTX_EDITRESPONSE, tw, "editresponse");
+  return tw;
 }
 
 char *owl_function_exec(int argc, const char *const *argv, const char *buff, int type)
@@ -2256,7 +2241,7 @@ void owl_function_show_filters(void)
     owl_fmtext_append_normal(&fm, "\n");
   }
   owl_function_popless_fmtext(&fm);
-  owl_fmtext_free(&fm);
+  owl_fmtext_cleanup(&fm);
 }
 
 void owl_function_show_filter(const char *name)
@@ -2300,30 +2285,38 @@ void owl_function_show_zpunts(void)
     owl_free(tmp);
   }
   owl_function_popless_fmtext(&fm);
-  owl_fmtext_free(&fm);
+  owl_fmtext_cleanup(&fm);
 }
 
 /* Create a filter for a class, instance if one doesn't exist.  If
  * instance is NULL then catch all messgaes in the class.  Returns the
  * name of the filter, which the caller must free.
+ * If 'related' is nonzero, encompass unclasses and .d classes as well.
  */
-char *owl_function_classinstfilt(const char *c, const char *i) 
+char *owl_function_classinstfilt(const char *c, const char *i, int related) 
 {
   owl_filter *f;
   char *argbuff, *filtname;
   char *tmpclass, *tmpinstance = NULL;
   char *class, *instance = NULL;
 
-  class = owl_util_baseclass(c);
-  if(i) {
-    instance = owl_util_baseclass(i);
+  if (related) {
+    class = owl_util_baseclass(c);
+    if (i) {
+      instance = owl_util_baseclass(i);
+    }
+  } else {
+    class = owl_strdup(c);
+    if (i) {
+      instance = owl_strdup(i);
+    }
   }
 
   /* name for the filter */
   if (!instance) {
-    filtname = owl_sprintf("class-%s", class);
+    filtname = owl_sprintf("%sclass-%s", related ? "related-" : "", class);
   } else {
-    filtname = owl_sprintf("class-%s-instance-%s", class, instance);
+    filtname = owl_sprintf("%sclass-%s-instance-%s", related ? "related-" : "", class, instance);
   }
   /* downcase it */
   {
@@ -2340,7 +2333,7 @@ char *owl_function_classinstfilt(const char *c, const char *i)
   
   /* if it already exists then go with it.  This lets users override */
   if (owl_global_get_filter(&g, filtname)) {
-    return(filtname);
+    goto done;
   }
 
   /* create the new filter */
@@ -2355,10 +2348,10 @@ char *owl_function_classinstfilt(const char *c, const char *i)
     owl_text_tr(tmpinstance, '"', '.');
   }
 
-  argbuff = owl_sprintf("class ^(un)*%s(\\.d)*$", tmpclass);
+  argbuff = owl_sprintf(related ? "class ^(un)*%s(\\.d)*$" : "class ^%s$", tmpclass);
   if (tmpinstance) {
     char *tmp = argbuff;
-    argbuff = owl_sprintf("%s and ( instance ^(un)*%s(\\.d)*$ )", tmp, tmpinstance);
+    argbuff = owl_sprintf(related ? "%s and ( instance ^(un)*%s(\\.d)*$ )" : "%s and instance ^%s$", tmp, tmpinstance);
     owl_free(tmp);
   }
   owl_free(tmpclass);
@@ -2370,6 +2363,7 @@ char *owl_function_classinstfilt(const char *c, const char *i)
   owl_global_add_filter(&g, f);
 
   owl_free(argbuff);
+done:
   owl_free(class);
   if (instance) {
     owl_free(instance);
@@ -2398,7 +2392,7 @@ char *owl_function_zuserfilt(const char *user)
 
   /* if it already exists then go with it.  This lets users override */
   if (owl_global_get_filter(&g, filtname)) {
-    return(owl_strdup(filtname));
+    return filtname;
   }
 
   /* create the new-internal filter */
@@ -2477,8 +2471,6 @@ char *owl_function_typefilt(const char *type)
   }
 
   /* create the new-internal filter */
-  f=owl_malloc(sizeof(owl_filter));
-
   esctype = owl_text_quote(type, OWL_REGEX_QUOTECHARS, OWL_REGEX_QUOTEWITH);
 
   argbuff = owl_sprintf("type ^%s$", esctype);
@@ -2531,13 +2523,14 @@ void owl_function_delete_curview_msgs(int flag)
  * If the curmsg is a personal AIM message returna  filter
  *    name to the AIM conversation with that user 
  */
-char *owl_function_smartfilter(int type)
+char *owl_function_smartfilter(int type, int invert_related)
 {
   const owl_view *v;
   const owl_message *m;
   char *zperson, *filtname=NULL;
-  const char *argv[1];
-  
+  const char *argv[2];
+  int related = owl_global_is_narrow_related(&g) ^ invert_related;
+
   v=owl_global_get_current_view(&g);
   m=owl_view_get_element(v, owl_global_get_curmsg(&g));
 
@@ -2581,24 +2574,23 @@ char *owl_function_smartfilter(int type)
 
     /* narrow class MESSAGE, instance foo, recip * messages to class, inst */
     if (!strcasecmp(owl_message_get_class(m), "message")) {
-      filtname=owl_function_classinstfilt(owl_message_get_class(m), owl_message_get_instance(m));
+      filtname=owl_function_classinstfilt(owl_message_get_class(m), owl_message_get_instance(m), related);
       return(filtname);
     }
 
     /* otherwise narrow to the class */
     if (type==0) {
-      filtname=owl_function_classinstfilt(owl_message_get_class(m), NULL);
+      filtname=owl_function_classinstfilt(owl_message_get_class(m), NULL, related);
     } else if (type==1) {
-      filtname=owl_function_classinstfilt(owl_message_get_class(m), owl_message_get_instance(m));
+      filtname=owl_function_classinstfilt(owl_message_get_class(m), owl_message_get_instance(m), related);
     }
     return(filtname);
   }
 
   /* pass it off to perl */
-  if(type) {
-    argv[0] = "-i";
-  };
-  return owl_perlconfig_message_call_method(m, "smartfilter", type ? 1 : 0, argv);
+  argv[0] = type ? "1" : "0";
+  argv[1] = related ? "1" : "0";
+  return owl_perlconfig_message_call_method(m, "smartfilter", 2, argv);
 }
 
 void owl_function_smartzpunt(int type)
@@ -2744,7 +2736,7 @@ void owl_function_show_colors(void)
   }
   
   owl_function_popless_fmtext(&fm);
-  owl_fmtext_free(&fm);
+  owl_fmtext_cleanup(&fm);
 }
 
 /* add the given class, inst, recip to the punt list for filtering.
@@ -2841,13 +2833,6 @@ void owl_function_punt(const char *filter, int direction)
   }
 }
 
-void owl_function_activate_keymap(const char *keymap)
-{
-  if (!owl_keyhandler_activate(owl_global_get_keyhandler(&g), keymap)) {
-    owl_function_error("Unable to activate keymap '%s'", keymap);
-  }
-}
-
 void owl_function_show_keymaps(void)
 {
   owl_list l;
@@ -2875,8 +2860,8 @@ void owl_function_show_keymaps(void)
   owl_fmtext_append_normal(&fm, "\n");
   
   owl_function_popless_fmtext(&fm);
-  owl_keyhandler_keymap_namelist_free(&l);
-  owl_fmtext_free(&fm);
+  owl_keyhandler_keymap_namelist_cleanup(&l);
+  owl_fmtext_cleanup(&fm);
 }
 
 char *owl_function_keymap_summary(const char *name)
@@ -2901,7 +2886,7 @@ void owl_function_show_keymap(const char *name)
     owl_fmtext_append_normal(&fm, "No such keymap...\n");
   }  
   owl_function_popless_fmtext(&fm);
-  owl_fmtext_free(&fm);
+  owl_fmtext_cleanup(&fm);
 }
 
 void owl_function_help_for_command(const char *cmdname)
@@ -2911,7 +2896,7 @@ void owl_function_help_for_command(const char *cmdname)
   owl_fmtext_init_null(&fm);
   owl_cmd_get_help(owl_global_get_cmddict(&g), cmdname, &fm);
   owl_function_popless_fmtext(&fm);  
-  owl_fmtext_free(&fm);
+  owl_fmtext_cleanup(&fm);
 }
 
 void owl_function_search_start(const char *string, int direction)
@@ -2922,7 +2907,7 @@ void owl_function_search_start(const char *string, int direction)
 
   if (string && owl_regex_create_quoted(&re, string) == 0) {
     owl_global_set_search_re(&g, &re);
-    owl_regex_free(&re);
+    owl_regex_cleanup(&re);
   } else {
     owl_global_set_search_re(&g, NULL);
   }
@@ -3018,7 +3003,7 @@ char *owl_function_ztext_stylestrip(const char *zt)
   owl_fmtext_init_null(&fm);
   owl_fmtext_append_ztext(&fm, zt);
   plaintext = owl_fmtext_print_plain(&fm);
-  owl_fmtext_free(&fm);
+  owl_fmtext_cleanup(&fm);
   return(plaintext);
 }
 
@@ -3119,7 +3104,7 @@ void owl_function_buddylist(int aim, int zephyr, const char *filename)
           }
         }
       }
-      owl_list_free_all(&anyone, owl_free);
+      owl_list_cleanup(&anyone, owl_free);
     }
   }
 #endif
@@ -3137,7 +3122,7 @@ void owl_function_buddylist(int aim, int zephyr, const char *filename)
   if(!interrupted) {
     owl_function_popless_fmtext(&fm);
   }
-  owl_fmtext_free(&fm);
+  owl_fmtext_cleanup(&fm);
 }
 
 /* Dump messages in the current view to the file 'filename'. */
@@ -3430,7 +3415,6 @@ void owl_function_makemsg(const char *fmt, ...)
   vsnprintf(buff, 2048, fmt, ap);
   owl_function_debugmsg("makemsg: %s", buff);
   waddstr(owl_global_get_curs_msgwin(&g), buff);  
-  wnoutrefresh(owl_global_get_curs_msgwin(&g));
   owl_global_set_needrefresh(&g);
   va_end(ap);
 }
@@ -3445,58 +3429,41 @@ void owl_function_zephyr_buddy_check(int notify)
 #ifdef HAVE_LIBZEPHYR
   int i, j;
   owl_list anyone;
-  owl_message *m;
   owl_zbuddylist *zbl;
+  GList **zaldlist;
+  GList *zaldptr;
+  ZAsyncLocateData_t *zald;
   const char *user;
-  ZLocations_t location[200];
-  int numlocs, ret;
 
   if (!owl_global_is_havezephyr(&g)) return;
+  owl_global_set_pseudologin_notify(&g, notify);
+  zbl = owl_global_get_zephyr_buddylist(&g);
+  zaldlist = owl_global_get_zaldlist(&g);
 
-  zbl=owl_global_get_zephyr_buddylist(&g);
+  /* Clear the existing ZALDs first. */
+  zaldptr = g_list_first(*zaldlist);
+  while (zaldptr) {
+    ZFreeALD(zaldptr->data);
+    owl_free(zaldptr->data);
+    zaldptr = g_list_next(zaldptr);
+  }
+  g_list_free(*zaldlist);
+  *zaldlist = NULL;
 
   owl_list_create(&anyone);
-  ret=owl_zephyr_get_anyone_list(&anyone, NULL);
-
-  j=owl_list_get_size(&anyone);
-  for (i=0; i<j; i++) {
-    user=owl_list_get_element(&anyone, i);
-    ret=ZLocateUser(zstr(user), &numlocs, ZAUTH);
-    if (ret!=ZERR_NONE) {
-      owl_function_error("Error getting location for %s", user);
-      continue;
-    }
-    numlocs=200;
-    ret=ZGetLocations(location, &numlocs);
-    if (ret==0) {
-      if ((numlocs>0) && !owl_zbuddylist_contains_user(zbl, user)) {
-	/* Send a PSEUDO LOGIN! */
-	if (notify) {
-	  m=owl_malloc(sizeof(owl_message));
-	  owl_message_create_pseudo_zlogin(m, 0, user, location[0].host, location[0].time, location[0].tty);
-	  owl_global_messagequeue_addmsg(&g, m);
-	}
-	owl_zbuddylist_adduser(zbl, user);
-	owl_function_debugmsg("owl_function_zephyr_buddy_check: login for %s ", user);
-      } else if ((numlocs==0) && owl_zbuddylist_contains_user(zbl, user)) {
-	/* I don't think this ever happens (if there are 0 locations we should get an error from
-	 * ZGetLocations)
-	 */
-	owl_function_error("owl_function_zephyr_buddy_check: exceptional case logout for %s ",user);
-      }
-    } else if ((ret==ZERR_NOLOCATIONS) && owl_zbuddylist_contains_user(zbl, user)) {
-      /* Send a PSEUDO LOGOUT! */
-      if (notify) {
-	m=owl_malloc(sizeof(owl_message));
-	owl_message_create_pseudo_zlogin(m, 1, user, "", "", "");
-	owl_global_messagequeue_addmsg(&g, m);
-      }
-      owl_zbuddylist_deluser(zbl, user);
-      owl_function_debugmsg("owl_function_zephyr_buddy_check: logout for %s ",user);
+  owl_zephyr_get_anyone_list(&anyone, NULL);
+  j = owl_list_get_size(&anyone);
+  for (i = 0; i < j; i++) {
+    user = owl_list_get_element(&anyone, i);
+    zald = owl_malloc(sizeof(ZAsyncLocateData_t));
+    if (ZRequestLocations(zstr(user), zald, UNACKED, ZAUTH) == ZERR_NONE) {
+      *zaldlist = g_list_append(*zaldlist, zald);
+    } else {
+      owl_free(zald);
     }
   }
 
-  owl_list_free_all(&anyone, owl_free);
+  owl_list_cleanup(&anyone, owl_free);
 #endif
 }
 
@@ -3518,7 +3485,7 @@ void owl_function_aimsearch_results(const char *email, owl_list *namelist)
   }
 
   owl_function_popless_fmtext(&fm);
-  owl_fmtext_free(&fm);
+  owl_fmtext_cleanup(&fm);
 }
 
 int owl_function_get_color_count(void)
