@@ -28,8 +28,9 @@ void owl_global_init(owl_global *g) {
     g->thishost=owl_strdup(hent->h_name);
   }
 
-  owl_context_init(&g->ctx);
-  owl_context_set_startup(&g->ctx);
+  g->context_stack = NULL;
+  owl_global_push_context(g, OWL_CTX_STARTUP, NULL, NULL);
+
   g->curmsg=0;
   g->topmsg=0;
   g->markedmsgid=-1;
@@ -43,8 +44,7 @@ void owl_global_init(owl_global *g) {
 
   g->rightshift=0;
 
-  g->tw = owl_editwin_allocate();
-  owl_editwin_init(g->tw, NULL, owl_global_get_typwin_lines(g), g->cols, OWL_EDITWIN_STYLE_ONELINE, NULL);
+  g->tw = NULL;
 
   owl_keyhandler_init(&g->kh);
   owl_keys_setup_keymaps(&g->kh);
@@ -56,7 +56,6 @@ void owl_global_init(owl_global *g) {
   owl_dict_create(&(g->styledict));
   g->curmsg_vert_offset=0;
   g->resizepending=0;
-  g->typwinactive=0;
   g->direction=OWL_DIRECTION_DOWNWARDS;
   g->zaway=0;
   if (has_colors()) {
@@ -109,6 +108,9 @@ void owl_global_init(owl_global *g) {
 
   owl_zbuddylist_create(&(g->zbuddies));
 
+  g->zaldlist = NULL;
+  g->pseudologin_notify = 0;
+
   owl_obarray_init(&(g->obarray));
 
   owl_message_init_fmtext_cache();
@@ -124,6 +126,30 @@ void owl_global_complete_setup(owl_global *g)
   owl_cmddict_setup(&(g->cmds));
 }
 
+/* If *pan does not exist, we create a new panel, otherwise we replace the
+   window in *pan with win.
+
+   libpanel PANEL objects cannot exist without owner a valid window. This
+   maintains the invariant for _owl_global_setup_windows. */
+void _owl_panel_set_window(PANEL **pan, WINDOW *win)
+{
+  WINDOW *oldwin;
+
+  if (win == NULL) {
+    owl_function_debugmsg("_owl_panel_set_window: passed NULL win (failed to allocate?)\n");
+    endwin();
+    exit(50);
+  }
+
+  if (*pan) {
+    oldwin = panel_window(*pan);
+    replace_panel(*pan, win);
+    delwin(oldwin);
+  } else {
+    *pan = new_panel(win);
+  }
+}
+
 void _owl_global_setup_windows(owl_global *g) {
   int cols, typwin_lines;
 
@@ -140,35 +166,66 @@ void _owl_global_setup_windows(owl_global *g) {
   owl_function_debugmsg("_owl_global_setup_windows: about to call newwin(%i, %i, 0, 0)\n", g->recwinlines, cols);
 
   /* create the new windows */
-  g->recwin=newwin(g->recwinlines, cols, 0, 0);
-  if (g->recwin==NULL) {
-    owl_function_debugmsg("_owl_global_setup_windows: newwin returned NULL\n");
-    endwin();
-    exit(50);
-  }
-      
-  g->sepwin=newwin(1, cols, g->recwinlines, 0);
-  g->msgwin=newwin(1, cols, g->recwinlines+1, 0);
-  g->typwin=newwin(typwin_lines, cols, g->recwinlines+2, 0);
+  _owl_panel_set_window(&g->recpan, newwin(g->recwinlines, cols, 0, 0));
+  _owl_panel_set_window(&g->seppan, newwin(1, cols, g->recwinlines, 0));
+  _owl_panel_set_window(&g->msgpan, newwin(1, cols, g->recwinlines+1, 0));
+  _owl_panel_set_window(&g->typpan, newwin(typwin_lines, cols, g->recwinlines+2, 0));
 
-  owl_editwin_set_curswin(g->tw, g->typwin, typwin_lines, g->cols);
+  if (g->tw)
+      owl_editwin_set_curswin(g->tw, owl_global_get_curs_typwin(g), typwin_lines, g->cols);
 
-  idlok(g->typwin, FALSE);
-  idlok(g->recwin, FALSE);
-  idlok(g->sepwin, FALSE);
-  idlok(g->msgwin, FALSE);
+  idlok(owl_global_get_curs_typwin(g), FALSE);
+  idlok(owl_global_get_curs_recwin(g), FALSE);
+  idlok(owl_global_get_curs_sepwin(g), FALSE);
+  idlok(owl_global_get_curs_msgwin(g), FALSE);
 
-  nodelay(g->typwin, 1);
-  keypad(g->typwin, TRUE);
-  wmove(g->typwin, 0, 0);
+  nodelay(owl_global_get_curs_typwin(g), 1);
+  keypad(owl_global_get_curs_typwin(g), TRUE);
+  wmove(owl_global_get_curs_typwin(g), 0, 0);
 
-  meta(g->typwin, TRUE);
+  meta(owl_global_get_curs_typwin(g), TRUE);
 }
 
 owl_context *owl_global_get_context(owl_global *g) {
-  return(&g->ctx);
+  if (!g->context_stack)
+    return NULL;
+  return g->context_stack->data;
 }
-			 
+
+static void owl_global_lookup_keymap(owl_global *g) {
+  owl_context *c = owl_global_get_context(g);
+  if (!c || !c->keymap)
+    return;
+
+  if (!owl_keyhandler_activate(owl_global_get_keyhandler(g), c->keymap)) {
+    owl_function_error("Unable to activate keymap '%s'", c->keymap);
+  }
+}
+
+void owl_global_push_context(owl_global *g, int mode, void *data, const char *keymap) {
+  owl_context *c;
+  if (!(mode & OWL_CTX_MODE_BITS))
+    mode |= OWL_CTX_INTERACTIVE;
+  c = owl_malloc(sizeof *c);
+  c->mode = mode;
+  c->data = data;
+  c->keymap = owl_strdup(keymap);
+  g->context_stack = g_list_prepend(g->context_stack, c);
+  owl_global_lookup_keymap(g);
+}
+
+void owl_global_pop_context(owl_global *g) {
+  owl_context *c;
+  if (!g->context_stack)
+    return;
+  c = owl_global_get_context(g);
+  g->context_stack = g_list_delete_link(g->context_stack,
+                                        g->context_stack);
+  owl_free(c->keymap);
+  owl_free(c);
+  owl_global_lookup_keymap(g);
+}
+
 int owl_global_get_lines(const owl_global *g) {
   return(g->lines);
 }
@@ -247,19 +304,19 @@ owl_keyhandler *owl_global_get_keyhandler(owl_global *g) {
 /* curses windows */
 
 WINDOW *owl_global_get_curs_recwin(const owl_global *g) {
-  return(g->recwin);
+  return panel_window(g->recpan);
 }
 
 WINDOW *owl_global_get_curs_sepwin(const owl_global *g) {
-  return(g->sepwin);
+  return panel_window(g->seppan);
 }
 
 WINDOW *owl_global_get_curs_msgwin(const owl_global *g) {
-  return(g->msgwin);
+  return panel_window(g->msgpan);
 }
 
 WINDOW *owl_global_get_curs_typwin(const owl_global *g) {
-  return(g->typwin);
+  return panel_window(g->typpan);
 }
 
 /* typwin */
@@ -268,24 +325,6 @@ owl_editwin *owl_global_get_typwin(const owl_global *g) {
   return(g->tw);
 }
 
-/* buffercommand */
-
-void owl_global_set_buffercommand(owl_global *g, const char *command) {
-  owl_editwin_set_command(owl_global_get_typwin(g), command);
-}
-
-const char *owl_global_get_buffercommand(const owl_global *g) {
-  return owl_editwin_get_command(owl_global_get_typwin(g));
-}
-
-void owl_global_set_buffercallback(owl_global *g, void (*cb)(owl_editwin*)) {
-  owl_editwin_set_callback(owl_global_get_typwin(g), cb);
-}
-
-void (*owl_global_get_buffercallback(const owl_global *g))(owl_editwin*) {
-  return owl_editwin_get_callback(owl_global_get_typwin(g));
-}
-
 /* refresh */
 
 int owl_global_is_needrefresh(const owl_global *g) {
@@ -325,25 +364,27 @@ int owl_global_get_rightshift(const owl_global *g) {
 
 /* typwin */
 
-int owl_global_is_typwin_active(const owl_global *g) {
-  if (g->typwinactive==1) return(1);
-  return(0);
-}
-
-void owl_global_set_typwin_active(owl_global *g) {
-  int d = owl_global_get_typewindelta(g);
-  if (d > 0)
+owl_editwin *owl_global_set_typwin_active(owl_global *g, int style, owl_history *hist) {
+  int d;
+  d = owl_global_get_typewindelta(g);
+  if (d > 0 && style == OWL_EDITWIN_STYLE_MULTILINE)
       owl_function_resize_typwin(owl_global_get_typwin_lines(g) + d);
 
-  g->typwinactive=1;
+  g->tw = owl_editwin_new(owl_global_get_curs_typwin(g),
+                          owl_global_get_typwin_lines(g),
+                          g->cols,
+                          style,
+                          hist);
+  return g->tw;
 }
 
 void owl_global_set_typwin_inactive(owl_global *g) {
   int d = owl_global_get_typewindelta(g);
-  if (d > 0)
+  if (d > 0 && owl_editwin_get_style(g->tw) == OWL_EDITWIN_STYLE_MULTILINE)
       owl_function_resize_typwin(owl_global_get_typwin_lines(g) - d);
 
-  g->typwinactive=0;
+  werase(owl_global_get_curs_typwin(g));
+  g->tw = NULL;
 }
 
 /* resize */
@@ -430,17 +471,7 @@ void owl_global_resize(owl_global *g, int x, int y) {
   struct winsize size;
     
   if (!g->resizepending) return;
-
-  /* delete the current windows */
-  delwin(g->recwin);
-  delwin(g->sepwin);
-  delwin(g->msgwin);
-  delwin(g->typwin);
-  if (!isendwin()) {
-    endwin();
-  }
-
-  refresh();
+  g->resizepending = 0;
 
   /* get the new size */
   ioctl(STDIN_FILENO, TIOCGWINSZ, &size);
@@ -464,9 +495,7 @@ void owl_global_resize(owl_global *g, int x, int y) {
     g->cols=y;
   }
 
-#ifdef HAVE_RESIZETERM
   resizeterm(size.ws_row, size.ws_col);
-#endif
 
   /* re-initialize the windows */
   _owl_global_setup_windows(g);
@@ -482,19 +511,15 @@ void owl_global_resize(owl_global *g, int x, int y) {
   g->needrefresh=1;
   owl_mainwin_redisplay(&(g->mw));
   sepbar(NULL);
-  owl_editwin_redisplay(g->tw, 0);
-  owl_function_full_redisplay();
+  if (g->tw)
+      owl_editwin_redisplay(g->tw);
+  else
+    werase(owl_global_get_curs_typwin(g));
 
-  /* TODO: this should handle other forms of popwins */
-  if (owl_popwin_is_active(owl_global_get_popwin(g)) 
-      && owl_global_get_viewwin(g)) {
-    owl_popwin_refresh(owl_global_get_popwin(g));
-    owl_viewwin_redisplay(owl_global_get_viewwin(g), 0);
-  }
+  owl_function_full_redisplay();
 
   owl_function_debugmsg("New size is %i lines, %i cols.", size.ws_row, size.ws_col);
   owl_function_makemsg("");
-  g->resizepending=0;
 }
 
 /* debug */
@@ -614,7 +639,8 @@ owl_filter *owl_global_get_filter(const owl_global *g, const char *name) {
   return NULL;
 }
 
-static void owl_global_free_filter_ent(void *data) {
+static void owl_global_delete_filter_ent(void *data)
+{
   owl_global_filter_ent *e = data;
   e->g->filterlist = g_list_remove(e->g->filterlist, e->f);
   owl_filter_delete(e->f);
@@ -627,14 +653,14 @@ void owl_global_add_filter(owl_global *g, owl_filter *f) {
   e->f = f;
 
   owl_dict_insert_element(&(g->filters), owl_filter_get_name(f),
-                          e, owl_global_free_filter_ent);
+                          e, owl_global_delete_filter_ent);
   g->filterlist = g_list_append(g->filterlist, f);
 }
 
 void owl_global_remove_filter(owl_global *g, const char *name) {
   owl_global_filter_ent *e = owl_dict_remove_element(&(g->filters), name);
   if (e)
-    owl_global_free_filter_ent(e);
+    owl_global_delete_filter_ent(e);
 }
 
 /* nextmsgid */
@@ -702,7 +728,7 @@ int owl_global_is_search_active(const owl_global *g) {
 
 void owl_global_set_search_re(owl_global *g, const owl_regex *re) {
   if (owl_regex_is_set(&g->search_re)) {
-    owl_regex_free(&g->search_re);
+    owl_regex_cleanup(&g->search_re);
     owl_regex_init(&g->search_re);
   }
   if (re != NULL)
@@ -925,6 +951,21 @@ owl_zbuddylist *owl_global_get_zephyr_buddylist(owl_global *g)
   return(&(g->zbuddies));
 }
 
+GList **owl_global_get_zaldlist(owl_global *g)
+{
+  return &(g->zaldlist);
+}
+
+int owl_global_get_pseudologin_notify(owl_global *g)
+{
+  return g->pseudologin_notify;
+}
+
+void owl_global_set_pseudologin_notify(owl_global *g, int notify)
+{
+  g->pseudologin_notify = notify;
+}
+
 struct termios *owl_global_get_startup_tio(owl_global *g)
 {
   return(&(g->startup_tio));
@@ -961,3 +1002,29 @@ void owl_global_set_interrupted(owl_global *g) {
 void owl_global_unset_interrupted(owl_global *g) {
   g->interrupted = 0;
 }
+
+FILE *owl_global_get_debug_file_handle(owl_global *g) {
+  static char *open_file = NULL;
+  const char *filename = owl_global_get_debug_file(g);
+  if (g->debug_file == NULL ||
+      (open_file && strcmp(filename, open_file) != 0)) {
+    char *path;
+    int fd;
+
+    if (g->debug_file)
+      fclose(g->debug_file);
+
+    g->debug_file = NULL;
+
+    path = owl_sprintf("%s.%d", filename, getpid());
+    fd = open(path, O_CREAT|O_WRONLY|O_EXCL, 0600);
+    owl_free(path);
+
+    if (fd >= 0)
+      g->debug_file = fdopen(fd, "a");
+
+    owl_free(open_file);
+    open_file = owl_strdup(filename);
+  }
+  return g->debug_file;
+}
