@@ -9,6 +9,7 @@
 #include <arpa/inet.h>
 #include <time.h>
 #include "owl.h"
+#include "filterproc.h"
 
 static owl_fmtext_cache fmtext_cache[OWL_FMTEXT_CACHE_SIZE];
 static owl_fmtext_cache * fmtext_cache_next = fmtext_cache;
@@ -519,8 +520,8 @@ void owl_message_curs_waddstr(owl_message *m, WINDOW *win, int aline, int bline,
 
   owl_fmtext_curs_waddstr(&b, win);
 
-  owl_fmtext_free(&a);
-  owl_fmtext_free(&b);
+  owl_fmtext_cleanup(&a);
+  owl_fmtext_cleanup(&b);
 }
 
 int owl_message_is_personal(const owl_message *m)
@@ -750,7 +751,7 @@ void owl_message_create_from_znotice(owl_message *m, const ZNotice_t *n)
   owl_message_set_direction_in(m);
   
   /* first save the full notice */
-  memcpy(&(m->notice), n, sizeof(ZNotice_t));
+  m->notice = *n;
 
   /* a little gross, we'll replace \r's with ' ' for now */
   owl_zephyr_hackaway_cr(&(m->notice));
@@ -837,16 +838,36 @@ void owl_message_create_from_znotice(owl_message *m, const ZNotice_t *n)
   }
   owl_free(tmp);
 
-#ifdef OWL_ENABLE_ZCRYPT
   /* if zcrypt is enabled try to decrypt the message */
   if (owl_global_is_zcrypt(&g) && !strcasecmp(n->z_opcode, "crypt")) {
-    char *out = owl_zcrypt_decrypt(owl_message_get_body(m), owl_message_get_class(m), owl_message_get_instance(m));
-    if (out) {
+    const char *argv[] = {
+      "zcrypt",
+      "-D",
+      "-c", owl_message_get_class(m),
+      "-i", owl_message_get_instance(m),
+      NULL
+    };
+    char *out;
+    int rv;
+    int status;
+    char *zcrypt;
+
+    zcrypt = owl_sprintf("%s/zcrypt", owl_get_bindir());
+
+    rv = call_filter(zcrypt, argv, owl_message_get_body(m), &out, &status);
+    owl_free(zcrypt);
+
+    if(!rv && !status) {
+      int len = strlen(out);
+      if(len >= 8 && !strcmp(out + len - 8, "**END**\n")) {
+        out[len - 8] = 0;
+      }
       owl_message_set_body(m, out);
       owl_free(out);
+    } else if(out) {
+      owl_free(out);
     }
   }
-#endif  
 }
 #else
 void owl_message_create_from_znotice(owl_message *m, const void *n)
@@ -943,7 +964,7 @@ void owl_message_create_from_zwrite(owl_message *m, const owl_zwrite *z, const c
   }
 }
 
-void owl_message_free(owl_message *m)
+void owl_message_cleanup(owl_message *m)
 {
   int i, j;
   owl_pair *p;
@@ -962,13 +983,13 @@ void owl_message_free(owl_message *m)
     owl_free(p);
   }
 
-  owl_list_free_simple(&(m->attributes));
+  owl_list_cleanup(&(m->attributes), NULL);
  
   owl_message_invalidate_format(m);
 }
 
 void owl_message_delete(owl_message *m)
 {
-  owl_message_free(m);
+  owl_message_cleanup(m);
   owl_free(m);
 }
