@@ -1,4 +1,4 @@
-/*  Copyright (c) 2006-2009 The BarnOwl Developers. All rights reserved.
+/*  Copyright (c) 2006-2010 The BarnOwl Developers. All rights reserved.
  *  Copyright (c) 2004 James Kretchmar. All rights reserved.
  *
  *  This program is free software. You can redistribute it and/or
@@ -302,13 +302,6 @@ int owl_process_messages(owl_ps_action *d, void *p)
     /* this should be optimized to not run if the new messages won't be displayed */
     owl_mainwin_redisplay(owl_global_get_mainwin(&g));
     sepbar(NULL);
-    if (owl_popwin_is_active(owl_global_get_popwin(&g))) {
-      owl_popwin_refresh(owl_global_get_popwin(&g));
-      /* TODO: this is a broken kludge */
-      if (owl_global_get_viewwin(&g)) {
-        owl_viewwin_redisplay(owl_global_get_viewwin(&g), 0);
-      }
-    }
     owl_global_set_needrefresh(&g);
   }
   return newmsgs;
@@ -472,12 +465,35 @@ void owl_zephyr_buddycheck_timer(owl_timer *t, void *data)
   }
 }
 
+static int owl_refresh_pre_select_action(owl_ps_action *a, void *data)
+{
+  /* if a resize has been scheduled, deal with it */
+  owl_global_resize(&g, 0, 0);
+
+  /* update the terminal if we need to */
+  if (owl_global_is_needrefresh(&g)) {
+    /* these are here in case a resize changes the windows */
+    WINDOW *sepwin = owl_global_get_curs_sepwin(&g);
+    WINDOW *typwin = owl_global_get_curs_typwin(&g);
+
+    /* push all changed windows to screen */
+    update_panels();
+    /* leave the cursor in the appropriate window */
+    if (!owl_popwin_is_active(owl_global_get_popwin(&g))
+	&& owl_global_get_typwin(&g)) {
+      owl_function_set_cursor(typwin);
+    } else {
+      owl_function_set_cursor(sepwin);
+    }
+    doupdate();
+    owl_global_set_noneedrefresh(&g);
+  }
+  return 0;
+}
+
 
 int main(int argc, char **argv, char **env)
 {
-  WINDOW *recwin, *sepwin, *typwin, *msgwin;
-  owl_editwin *tw;
-  owl_popwin *pw;
   int argcsave;
   const char *const *argvsave;
   char *perlout, *perlerr;
@@ -498,8 +514,6 @@ int main(int argc, char **argv, char **env)
   owl_parse_options(argc, argv, &opts);
   g.load_initial_subs = opts.load_initial_subs;
 
-  owl_function_debugmsg("startup: Finished parsing arguments");
-
   owl_register_signal_handlers();
   owl_start_curses();
 
@@ -539,7 +553,10 @@ int main(int argc, char **argv, char **env)
 
   /* Initialize perl */
   owl_function_debugmsg("startup: processing config file");
-  owl_context_set_readconfig(owl_global_get_context(&g));
+
+  owl_global_pop_context(&g);
+  owl_global_push_context(&g, OWL_CTX_READCONFIG, NULL, NULL);
+
   perlerr=owl_perlconfig_initperl(opts.configfile, &argc, &argv, &env);
   if (perlerr) {
     endwin();
@@ -569,13 +586,6 @@ int main(int argc, char **argv, char **env)
   perlout = owl_perlconfig_execute("BarnOwl::Hooks::_startup();");
   if (perlout) owl_free(perlout);
 
-  /* hold on to the window names for convenience */
-  msgwin=owl_global_get_curs_msgwin(&g);
-  recwin=owl_global_get_curs_recwin(&g);
-  sepwin=owl_global_get_curs_sepwin(&g);
-  typwin=owl_global_get_curs_typwin(&g);
-  tw=owl_global_get_typwin(&g);
-
   /* welcome message */
   owl_function_debugmsg("startup: creating splash message");
   owl_function_adminmsg("",
@@ -595,7 +605,7 @@ int main(int argc, char **argv, char **env)
   owl_function_debugmsg("startup: processing startup file");
   owl_function_source(NULL);
 
-  wrefresh(sepwin);
+  update_panels();
 
   /* Set the default style */
   owl_function_debugmsg("startup: setting startup and default style");
@@ -613,52 +623,21 @@ int main(int argc, char **argv, char **env)
       owl_function_error("No such style: %s", owl_global_get_default_style(&g));
 
   owl_function_debugmsg("startup: setting context interactive");
-  owl_context_set_interactive(owl_global_get_context(&g));
+
+  owl_global_pop_context(&g);
+  owl_global_push_context(&g, OWL_CTX_READCONFIG|OWL_CTX_RECV, NULL, "recv");
 
   owl_select_add_timer(180, 180, owl_zephyr_buddycheck_timer, NULL, NULL);
 
   /* If we ever deprecate the mainloop hook, remove this. */
   owl_select_add_timer(0, 1, owl_perlconfig_mainloop, NULL, NULL);
 
+  owl_select_add_pre_select_action(owl_refresh_pre_select_action, NULL, NULL);
   owl_select_add_pre_select_action(owl_process_messages, NULL, NULL);
 
   owl_function_debugmsg("startup: entering main loop");
   /* main loop */
   while (1) {
-
-    /* if a resize has been scheduled, deal with it */
-    owl_global_resize(&g, 0, 0);
-
-    /* these are here in case a resize changes the windows */
-    msgwin=owl_global_get_curs_msgwin(&g);
-    recwin=owl_global_get_curs_recwin(&g);
-    sepwin=owl_global_get_curs_sepwin(&g);
-    typwin=owl_global_get_curs_typwin(&g);
-
-    /* if a popwin just came up, refresh it */
-    pw=owl_global_get_popwin(&g);
-    if (owl_popwin_is_active(pw) && owl_popwin_needs_first_refresh(pw)) {
-      owl_popwin_refresh(pw);
-      owl_popwin_no_needs_first_refresh(pw);
-      owl_global_set_needrefresh(&g);
-      /* TODO: this is a broken kludge */
-      if (owl_global_get_viewwin(&g)) {
-	owl_viewwin_redisplay(owl_global_get_viewwin(&g), 0);
-      }
-    }
-
-    /* update the terminal if we need to */
-    if (owl_global_is_needrefresh(&g)) {
-      /* leave the cursor in the appropriate window */
-      if (owl_global_is_typwin_active(&g)) {
-	owl_function_set_cursor(typwin);
-      } else {
-	owl_function_set_cursor(sepwin);
-      }
-      doupdate();
-      owl_global_set_noneedrefresh(&g);
-    }
-
     /* select on FDs we know about. */
     owl_select();
 
