@@ -1,4 +1,4 @@
-/*  Copyright (c) 2006-2009 The BarnOwl Developers. All rights reserved.
+/*  Copyright (c) 2006-2010 The BarnOwl Developers. All rights reserved.
  *  Copyright (c) 2004 James Kretchmar. All rights reserved.
  *
  *  This program is free software. You can redistribute it and/or
@@ -9,8 +9,21 @@
 #ifndef INC_OWL_H
 #define INC_OWL_H
 
+#ifdef HAVE_STDBOOL_H
+#include <stdbool.h>
+#else
+#ifndef HAVE__BOOL
+#define _Bool signed char
+#endif
+#define bool _Bool
+#define false 0
+#define true 1
+#define __bool_true_false_are_defined 1
+#endif  /* HAVE_STDBOOL_H */
+
 #ifndef OWL_PERL
 #include <curses.h>
+#include <panel.h>
 #endif
 #include <sys/param.h>
 #include <EXTERN.h>
@@ -33,9 +46,9 @@
 /* Perl and curses don't play nice. */
 #ifdef OWL_PERL
 typedef void WINDOW;
+typedef void PANEL;
 /* logout is defined in FreeBSD. */
 #define logout logout_
-/* aim.h defines bool */
 #define HAS_BOOL
 #include <perl.h>
 #include "owl_perl.h"
@@ -60,7 +73,7 @@ typedef void HV;
 #define OWL_STDERR_REDIR 1
 
 #define OWL_DEBUG 0
-#define OWL_DEBUG_FILE "/var/tmp/owldebug"
+#define OWL_DEBUG_FILE "/var/tmp/barnowl-debug"
 
 #define OWL_CONFIG_DIR "/.owl"             /* this is relative to the user's home directory */
 #define OWL_STARTUP_FILE "/.owl/startup"   /* this is relative to the user's home directory */
@@ -249,7 +262,7 @@ typedef struct _owl_variable {
 			  char *buf, int bufsize, const void *val); 
                                 /* converts val to a string 
 				 * and puts into buf */
-  void  (*free_fn)(struct _owl_variable *v);
+  void (*delete_fn)(struct _owl_variable *v);
 				/* frees val as needed */
 } owl_variable;
 
@@ -289,6 +302,7 @@ typedef owl_dict owl_cmddict;	/* dict of commands */
 typedef struct _owl_context {
   int   mode;
   void *data;		/* determined by mode */
+  char *keymap;
 } owl_context;
 
 typedef struct _owl_cmd {	/* command */
@@ -389,12 +403,11 @@ typedef struct _owl_viewwin {
 } owl_viewwin;
   
 typedef struct _owl_popwin {
-  WINDOW *borderwin;
-  WINDOW *popwin;
+  PANEL *borderpanel;
+  PANEL *poppanel;
   int lines;
   int cols;
   int active;
-  int needsfirstrefresh;
 } owl_popwin;
 
 typedef struct _owl_messagelist {
@@ -543,7 +556,7 @@ typedef struct _owl_global {
   owl_list puntlist;
   owl_vardict vars;
   owl_cmddict cmds;
-  owl_context ctx;
+  GList *context_stack;
   owl_errqueue errqueue;
   int lines, cols;
   int curmsg, topmsg;
@@ -551,12 +564,11 @@ typedef struct _owl_global {
   int curmsg_vert_offset;
   owl_view current_view;
   owl_messagelist msglist;
-  WINDOW *recwin, *sepwin, *msgwin, *typwin;
+  PANEL *recpan, *seppan, *msgpan, *typpan;
   int needrefresh;
   int rightshift;
   int resizepending;
   int recwinlines;
-  int typwinactive;
   char *thishost;
   char *homedir;
   char *confdir;
@@ -601,6 +613,8 @@ typedef struct _owl_global {
   siginfo_t err_signal_info;
   owl_zbuddylist zbuddies;
   owl_timer zephyr_buddycheck_timer;
+  GList *zaldlist;
+  int pseudologin_notify;
   struct termios startup_tio;
   owl_obarray obarray;
   owl_list io_dispatch_list;
@@ -609,6 +623,7 @@ typedef struct _owl_global {
   owl_timer *aim_nop_timer;
   int load_initial_subs;
   int interrupted;
+  FILE *debug_file;
 } owl_global;
 
 /* globals */
