@@ -136,8 +136,43 @@ sub skip_msg {
     return grep {$_ eq $type} split ' ', $skip;
 }
 
-use constant OPTIONAL_CHANNEL => 1;
-use constant REQUIRE_CHANNEL => 2;
+=head2 mk_irc_command SUB FLAGS
+
+Return a subroutine that can be bound as a an IRC command. The
+subroutine will be called with arguments (COMMAND-NAME,
+IRC-CONNECTION, [CHANNEL], ARGV...).
+
+C<IRC-CONNECTION> and C<CHANNEL> will be inferred from arguments to
+the command and the current message if appropriate.
+
+The bitwise C<or> of zero or more C<FLAGS> can be passed in as a
+second argument to alter the behavior of the returned commands:
+
+=over 4
+
+=item C<CHANNEL_ARG>
+
+This command accepts the name of a channel. Pass in the C<CHANNEL>
+argument listed above, and die if no channel argument can be found.
+
+=item C<CHANNEL_OPTIONAL>
+
+Pass the channel argument, but don't die if not present. Only relevant
+with C<CHANNEL_ARG>.
+
+=item C<ALLOW_DISCONNECTED>
+
+C<IRC-CONNECTION> may be a disconnected connection object that is
+currently pending a reconnect.
+
+=back
+
+=cut
+
+use constant CHANNEL_ARG        => 1;
+use constant CHANNEL_OPTIONAL   => 2;
+
+use constant ALLOW_DISCONNECTED => 4;
 
 sub register_commands {
     BarnOwl::new_command(
@@ -166,7 +201,7 @@ END_DESCR
     );
 
     BarnOwl::new_command(
-        'irc-disconnect' => \&cmd_disconnect,
+        'irc-disconnect' => mk_irc_command( \&cmd_disconnect, ALLOW_DISCONNECTED ),
         {
             summary => 'Disconnect from an IRC server',
             usage   => 'irc-disconnect [-a ALIAS]',
@@ -191,7 +226,7 @@ END_DESCR
     );
 
     BarnOwl::new_command(
-        'irc-mode' => mk_irc_command( \&cmd_mode, OPTIONAL_CHANNEL ),
+        'irc-mode' => mk_irc_command( \&cmd_mode, CHANNEL_OPTIONAL|CHANNEL_ARG ),
         {
             summary => 'Change an IRC channel or user mode',
             usage   => 'irc-mode [-a ALIAS] TARGET [+-]MODE OPTIONS',
@@ -215,7 +250,7 @@ END_DESCR
     );
 
     BarnOwl::new_command(
-        'irc-part' => mk_irc_command( \&cmd_part, REQUIRE_CHANNEL ),
+        'irc-part' => mk_irc_command( \&cmd_part, CHANNEL_ARG ),
         {
             summary => 'Leave an IRC channel',
             usage   => 'irc-part [-a ALIAS] #channel',
@@ -240,7 +275,7 @@ END_DESCR
     );
 
     BarnOwl::new_command(
-        'irc-names' => mk_irc_command( \&cmd_names, REQUIRE_CHANNEL ),
+        'irc-names' => mk_irc_command( \&cmd_names, CHANNEL_ARG ),
         {
             summary => 'View the list of users in a channel',
             usage   => 'irc-names [-a ALIAS] #channel',
@@ -293,7 +328,7 @@ END_DESCR
     BarnOwl::new_command( 'irc-stats' => mk_irc_command( \&cmd_stats ) );
 
     BarnOwl::new_command(
-        'irc-topic' => mk_irc_command( \&cmd_topic, REQUIRE_CHANNEL ),
+        'irc-topic' => mk_irc_command( \&cmd_topic, CHANNEL_ARG ),
         {
             summary => 'View or change the topic of an IRC channel',
             usage   => 'irc-topic [-a ALIAS] #channel [TOPIC]',
@@ -390,27 +425,15 @@ sub cmd_connect {
 }
 
 sub cmd_disconnect {
-    # Such a hack
-    local *get_connection_by_alias = sub {
-        my $key = shift;
-        return $ircnets{$key} if exists $ircnets{$key};
-        return $reconnect{$key}{conn} if exists $reconnect{$key};
-        die("No such ircnet: $key\n");
-    };
-
-    mk_irc_command(
-        sub {
-            my $cmd = shift;
-            my $conn = shift;
-            if ($conn->conn->connected) {
-                $conn->conn->disconnect;
-            } elsif ($reconnect{$conn->alias}) {
-                BarnOwl::admin_message('IRC',
-                                       "[" . $conn->alias . "] Reconnect cancelled");
-                delete $reconnect{$conn->alias};
-            }
-        }
-    )->(@_);
+    my $cmd = shift;
+    my $conn = shift;
+    if ($conn->conn->connected) {
+        $conn->conn->disconnect;
+    } elsif ($reconnect{$conn->alias}) {
+        BarnOwl::admin_message('IRC',
+                               "[" . $conn->alias . "] Reconnect cancelled");
+        $conn->cancel_reconnect;
+    }
 }
 
 sub cmd_msg {
@@ -429,29 +452,35 @@ sub cmd_msg {
 sub process_msg {
     my $conn = shift;
     my $to = shift;
-    my $body = shift;
-    # Strip whitespace. In the future -- send one message/line?
-    $body =~ tr/\n\r/  /;
-    if ($body =~ /^\/me (.*)/) {
-        $conn->conn->me($to, Encode::encode('utf-8', $1));
-        $body = '* '.$conn->nick.' '.$1;
-    } else {
-        $conn->conn->privmsg($to, Encode::encode('utf-8', $body));
+    my $fullbody = shift;
+    my @msgs;
+    # Require the user to send in paragraphs (double-newline between) to
+    # actually send multiple PRIVMSGs, in order to play nice with autofill.
+    $fullbody =~ s/\r//g;
+    @msgs = split "\n\n", $fullbody;
+    map { tr/\n/ / } @msgs;
+    for my $body (@msgs) {
+	if ($body =~ /^\/me (.*)/) {
+	    $conn->conn->me($to, Encode::encode('utf-8', $1));
+	    $body = '* '.$conn->nick.' '.$1;
+	} else {
+	    $conn->conn->privmsg($to, Encode::encode('utf-8', $body));
+	}
+	my $msg = BarnOwl::Message->new(
+	    type        => 'IRC',
+	    direction   => is_private($to) ? 'out' : 'in',
+	    server      => $conn->server,
+	    network     => $conn->alias,
+	    recipient   => $to,
+	    body        => $body,
+	    sender      => $conn->nick,
+	    is_private($to) ?
+	      (isprivate  => 'true') : (channel => $to),
+	    replycmd    => BarnOwl::quote('irc-msg',  '-a', $conn->alias, $to),
+	    replysendercmd => BarnOwl::quote('irc-msg', '-a', $conn->alias, $to),
+	);
+	BarnOwl::queue_message($msg);
     }
-    my $msg = BarnOwl::Message->new(
-        type        => 'IRC',
-        direction   => is_private($to) ? 'out' : 'in',
-        server      => $conn->server,
-        network     => $conn->alias,
-        recipient   => $to,
-        body        => $body,
-        sender      => $conn->nick,
-        is_private($to) ?
-          (isprivate  => 'true') : (channel => $to),
-        replycmd    => BarnOwl::quote('irc-msg',  '-a', $conn->alias, $to),
-        replysendercmd => BarnOwl::quote('irc-msg', '-a', $conn->alias, $to),
-       );
-    BarnOwl::queue_message($msg);
     return;
 }
 
@@ -563,7 +592,7 @@ sub cmd_quote {
 
 sub mk_irc_command {
     my $sub = shift;
-    my $use_channel = shift || 0;
+    my $flags = shift || 0;
     return sub {
         my $cmd = shift;
         my $conn;
@@ -577,9 +606,10 @@ sub mk_irc_command {
         $getopt->getoptions("alias=s" => \$alias);
 
         if(defined($alias)) {
-            $conn = get_connection_by_alias($alias);
+            $conn = get_connection_by_alias($alias,
+                                            $flags & ALLOW_DISCONNECTED);
         }
-        if($use_channel) {
+        if($flags & CHANNEL_ARG) {
             $channel = $ARGV[0];
             if(defined($channel) && $channel =~ /^#/) {
                 if($channels{$channel} && @{$channels{$channel}} == 1) {
@@ -593,12 +623,15 @@ sub mk_irc_command {
             }
         }
 
-        if(!$channel && $use_channel == REQUIRE_CHANNEL) {
+        if(!$channel &&
+           ($flags & CHANNEL_ARG) &&
+           !($flags & CHANNEL_OPTIONAL)) {
             die("Usage: $cmd <channel>\n");
         }
         if(!$conn) {
             if($m && $m->type eq 'IRC') {
-                $conn = get_connection_by_alias($m->network);
+                $conn = get_connection_by_alias($m->network,
+                                               $flags & ALLOW_DISCONNECTED);
             }
         }
         if(!$conn && scalar keys %ircnets == 1) {
@@ -607,7 +640,7 @@ sub mk_irc_command {
         if(!$conn) {
             die("You must specify an IRC network using -a.\n");
         }
-        if($use_channel) {
+        if($flags & CHANNEL_ARG) {
             $sub->($cmd, $conn, $channel, @ARGV);
         } else {
             $sub->($cmd, $conn, @ARGV);
@@ -617,8 +650,11 @@ sub mk_irc_command {
 
 sub get_connection_by_alias {
     my $key = shift;
-    die("No such ircnet: $key\n") unless exists $ircnets{$key};
-    return $ircnets{$key};
+    my $allow_disconnected = shift;
+
+    return $ircnets{$key} if exists $ircnets{$key};
+    return $reconnect{$key} if $allow_disconnected && exists $reconnect{$key};
+    die("No such ircnet: $key\n")
 }
 
 1;
