@@ -119,7 +119,6 @@ void sepbar(const char *in)
     
   wattroff(sepwin, A_BOLD);
   wattroff(sepwin, A_REVERSE);
-  wnoutrefresh(sepwin);
 }
 
 char **atokenize(const char *buffer, const char *sep, int *i)
@@ -129,8 +128,7 @@ char **atokenize(const char *buffer, const char *sep, int *i)
   char *workbuff, *foo;
   int done=0, first=1, count=0;
 
-  workbuff=owl_malloc(strlen(buffer)+1);
-  memcpy(workbuff, buffer, strlen(buffer)+1);
+  workbuff = owl_strdup(buffer);
 
   args=NULL;
   while (!done) {
@@ -238,7 +236,7 @@ char *owl_util_makepath(const char *in)
   return(out);
 }
 
-void atokenize_free(char **tok, int nels)
+void atokenize_delete(char **tok, int nels)
 {
   int i;
   for (i=0; i<nels; i++) {
@@ -248,7 +246,7 @@ void atokenize_free(char **tok, int nels)
 }
 
 
-void owl_parsefree(char **argv, int argc)
+void owl_parse_delete(char **argv, int argc)
 {
   int i;
 
@@ -337,7 +335,7 @@ char **owl_parseline(const char *line, int *argc)
 
   /* check for unbalanced quotes */
   if (quote!='\0') {
-    owl_parsefree(argv, *argc);
+    owl_parse_delete(argv, *argc);
     *argc=-1;
     return(NULL);
   }
@@ -610,6 +608,14 @@ const char * owl_get_datadir(void)
   return DATADIR;
 }
 
+const char * owl_get_bindir(void)
+{
+  const char * bindir = getenv("BARNOWL_BIN_DIR");
+  if(bindir != NULL)
+    return bindir;
+  return BINDIR;
+}
+
 /* Strips format characters from a valid utf-8 string. Returns the
    empty string if 'in' does not validate. */
 char * owl_strip_format_chars(const char *in)
