@@ -178,8 +178,15 @@ int owl_zephyr_shutdown(void)
 int owl_zephyr_zpending(void)
 {
 #ifdef HAVE_LIBZEPHYR
-  if(owl_global_is_havezephyr(&g))
-    return(ZPending());
+  Code_t code;
+  if(owl_global_is_havezephyr(&g)) {
+    if((code = ZPending()) < 0) {
+      owl_function_debugmsg("Error (%s) in ZPending()\n",
+                            error_message(code));
+      return 0;
+    }
+    return code;
+  }
 #endif
   return 0;
 }
@@ -257,6 +264,7 @@ int owl_zephyr_loadsubs(const char *filename, int error_on_nofile)
   subsfile = owl_zephyr_dotfile(".zephyr.subs", filename);
 
   if (stat(subsfile, &statbuff) != 0) {
+    owl_free(subsfile);
     if (error_on_nofile == 1)
       return -1;
     return 0;
@@ -372,8 +380,10 @@ int owl_zephyr_loadloginsubs(const char *filename)
   subs = owl_malloc(numSubs * sizeof(ZSubscription_t));
   subsfile = owl_zephyr_dotfile(".anyone", filename);
 
-  if (stat(subsfile, &statbuff) == -1)
+  if (stat(subsfile, &statbuff) == -1) {
+    owl_free(subsfile);
     return 0;
+  }
 
   ZResetAuthentication();
   count = 0;
@@ -814,7 +824,7 @@ void owl_zephyr_handle_ack(const ZNotice_t *retnotice)
 
       owl_log_outgoing_zephyr_error(&zw, buff);
 
-      owl_zwrite_free(&zw);
+      owl_zwrite_cleanup(&zw);
       owl_free(tmp);
     }
   } else {
@@ -1298,6 +1308,71 @@ int owl_zephyr_get_anyone_list(owl_list *in, const char *filename)
 #endif
 }
 
+#ifdef HAVE_LIBZEPHYR
+void owl_zephyr_process_pseudologin(ZNotice_t *n)
+{
+  owl_message *m;
+  owl_zbuddylist *zbl;
+  GList **zaldlist;
+  GList *zaldptr;
+  ZAsyncLocateData_t *zald = NULL;
+  ZLocations_t location;
+  int numlocs, ret, notify;
+
+  /* Find a ZALD to match this notice. */
+  zaldlist = owl_global_get_zaldlist(&g);
+  zaldptr = g_list_first(*zaldlist);
+  while (zaldptr) {
+    if (ZCompareALDPred(n, zaldptr->data)) {
+      zald = zaldptr->data;
+      *zaldlist = g_list_remove(*zaldlist, zaldptr->data);
+      break;
+    }
+    zaldptr = g_list_next(zaldptr);
+  }
+  if (zald) {
+    /* Deal with notice. */
+    notify = owl_global_get_pseudologin_notify(&g);
+    zbl = owl_global_get_zephyr_buddylist(&g);
+    ret = ZParseLocations(n, zald, &numlocs, NULL);
+    if (ret == ZERR_NONE) {
+      if (numlocs > 0 && !owl_zbuddylist_contains_user(zbl, zald->user)) {
+        if (notify) {
+          numlocs = 1;
+          ret = ZGetLocations(&location, &numlocs);
+          if (ret == ZERR_NONE) {
+            /* Send a PSEUDO LOGIN! */
+            m = owl_malloc(sizeof(owl_message));
+            owl_message_create_pseudo_zlogin(m, 0, zald->user,
+                                             location.host,
+                                             location.time,
+                                             location.tty);
+            owl_global_messagequeue_addmsg(&g, m);
+          }
+          owl_zbuddylist_adduser(zbl, zald->user);
+          owl_function_debugmsg("owl_function_zephyr_buddy_check: login for %s ", zald->user);
+        }
+      } else if (numlocs == 0 && owl_zbuddylist_contains_user(zbl, zald->user)) {
+        /* Send a PSEUDO LOGOUT! */
+        if (notify) {
+          m = owl_malloc(sizeof(owl_message));
+          owl_message_create_pseudo_zlogin(m, 1, zald->user, "", "", "");
+          owl_global_messagequeue_addmsg(&g, m);
+        }
+        owl_zbuddylist_deluser(zbl, zald->user);
+        owl_function_debugmsg("owl_function_zephyr_buddy_check: logout for %s ", zald->user);
+      }
+    }
+    ZFreeALD(zald);
+    owl_free(zald);
+  }
+}
+#else
+void owl_zephyr_process_pseudologin(void *n)
+{
+}
+#endif
+
 /*
  * Process zephyrgrams from libzephyr's queue. To prevent starvation,
  * process a maximum of OWL_MAX_ZEPHYRGRAMS_TO_PROCESS.
@@ -1312,11 +1387,16 @@ static int _owl_zephyr_process_events(void)
   int zpendcount=0;
 #ifdef HAVE_LIBZEPHYR
   ZNotice_t notice;
+  Code_t code;
   owl_message *m=NULL;
 
   while(owl_zephyr_zpending() && zpendcount < OWL_MAX_ZEPHYRGRAMS_TO_PROCESS) {
     if (owl_zephyr_zpending()) {
-      ZReceiveNotice(&notice, NULL);
+      if ((code = ZReceiveNotice(&notice, NULL)) != ZERR_NONE) {
+        owl_function_debugmsg("Error: %s while calling ZReceiveNotice\n",
+                              error_message(code));
+        continue;
+      }
       zpendcount++;
 
       /* is this an ack from a zephyr we sent? */
@@ -1332,6 +1412,13 @@ static int _owl_zephyr_process_events(void)
         continue;
       }
 
+      /* if it is a LOCATE message, it's for pseudologins. */
+      if (strcmp(notice.z_opcode, LOCATE_LOCATE) == 0) {
+        owl_zephyr_process_pseudologin(&notice);
+        ZFreeNotice(&notice);
+        continue;
+      }
+
       /* create the new message */
       m=owl_malloc(sizeof(owl_message));
       owl_message_create_from_znotice(m, &notice);
